@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -60,7 +60,7 @@ void db_deleter_by_type_and_id_t::delete_rows(std::string const &table,
 
         auto const pos = column.find(',');
         assert(pos != std::string::npos);
-        std::string type = column.substr(0, pos);
+        std::string const type = column.substr(0, pos);
 
         fmt::format_to(std::back_inserter(sql),
                        ") AS t (osm_type, osm_id) WHERE"
@@ -85,7 +85,7 @@ db_copy_thread_t::db_copy_thread_t(std::string const &conninfo)
 {
     // conninfo is captured by copy here, because we don't know wether the
     // reference will still be valid once we get around to running the thread
-    m_worker = std::thread{thread_t{conninfo, m_shared}};
+    m_worker = std::thread{thread_t{conninfo, &m_shared}};
 }
 
 db_copy_thread_t::~db_copy_thread_t() { finish(); }
@@ -106,7 +106,7 @@ void db_copy_thread_t::add_buffer(std::unique_ptr<db_cmd_t> &&buffer)
 void db_copy_thread_t::sync_and_wait()
 {
     std::promise<void> barrier;
-    std::future<void> sync = barrier.get_future();
+    std::future<void> const sync = barrier.get_future();
     add_buffer(std::make_unique<db_cmd_sync_t>(std::move(barrier)));
     sync.wait();
 }
@@ -119,7 +119,7 @@ void db_copy_thread_t::finish()
     }
 }
 
-db_copy_thread_t::thread_t::thread_t(std::string conninfo, shared &shared)
+db_copy_thread_t::thread_t::thread_t(std::string conninfo, shared *shared)
 : m_conninfo(std::move(conninfo)), m_shared(shared)
 {}
 
@@ -139,13 +139,13 @@ void db_copy_thread_t::thread_t::operator()()
         while (!done) {
             std::unique_ptr<db_cmd_t> item;
             {
-                std::unique_lock<std::mutex> lock{m_shared.queue_mutex};
-                m_shared.queue_cond.wait(
-                    lock, [&] { return !m_shared.worker_queue.empty(); });
+                std::unique_lock<std::mutex> lock{m_shared->queue_mutex};
+                m_shared->queue_cond.wait(
+                    lock, [&] { return !m_shared->worker_queue.empty(); });
 
-                item = std::move(m_shared.worker_queue.front());
-                m_shared.worker_queue.pop_front();
-                m_shared.queue_full_cond.notify_one();
+                item = std::move(m_shared->worker_queue.front());
+                m_shared->worker_queue.pop_front();
+                m_shared->queue_full_cond.notify_one();
             }
 
             switch (item->type) {
@@ -184,7 +184,7 @@ void db_copy_thread_t::thread_t::write_to_db(db_cmd_copy_t *buffer)
         start_copy(buffer->target);
     }
 
-    m_conn->copy_data(buffer->buffer, buffer->target->name);
+    m_conn->copy_send(buffer->buffer, buffer->target->name);
 }
 
 void db_copy_thread_t::thread_t::start_copy(
@@ -205,7 +205,7 @@ void db_copy_thread_t::thread_t::start_copy(
     }
 
     sql.push_back('\0');
-    m_conn->query(PGRES_COPY_IN, sql.data());
+    m_conn->copy_start(sql.data());
 
     m_inflight = target;
 }
@@ -213,7 +213,7 @@ void db_copy_thread_t::thread_t::start_copy(
 void db_copy_thread_t::thread_t::finish_copy()
 {
     if (m_inflight) {
-        m_conn->end_copy(m_inflight->name);
+        m_conn->copy_end(m_inflight->name);
         m_inflight.reset();
     }
 }
