@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -15,10 +15,13 @@
  * http://subversion.nexusuk.org/projects/openpistemap/trunk/scripts/expire_tiles.py
  */
 
+#include <algorithm>
 #include <cerrno>
 #include <cmath>
+#include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <stdexcept>
 #include <string>
 
 #include "expire-tiles.hpp"
@@ -34,44 +37,12 @@
 // How many tiles worth of space to leave either side of a changed feature
 static constexpr double const tile_expiry_leeway = 0.1;
 
-tile_output_t::tile_output_t(char const *filename)
-: outfile(fopen(filename, "a"))
-{
-    if (outfile == nullptr) {
-        log_warn("Failed to open expired tiles file ({}).  Tile expiry "
-                 "list will not be written!",
-                 std::strerror(errno));
-    }
-}
-
-tile_output_t::~tile_output_t()
-{
-    if (outfile) {
-        fclose(outfile);
-    }
-}
-
-void tile_output_t::output_dirty_tile(tile_t const &tile)
-{
-    if (!outfile) {
-        return;
-    }
-
-    fmt::print(outfile, "{}/{}/{}\n", tile.zoom(), tile.x(), tile.y());
-}
-
 expire_tiles::expire_tiles(uint32_t max_zoom, double max_bbox,
                            std::shared_ptr<reprojection> projection)
 : m_projection(std::move(projection)), m_max_bbox(max_bbox),
   m_maxzoom(max_zoom), m_map_width(1U << m_maxzoom)
 {}
 
-void expire_tiles::output_and_destroy(char const *filename, uint32_t minzoom)
-{
-    tile_output_t output_writer{filename};
-    output_and_destroy<tile_output_t>(output_writer, minzoom);
-}
-
 void expire_tiles::expire_tile(uint32_t x, uint32_t y)
 {
     // Only try to insert to tile into the set if the last inserted tile
@@ -107,45 +78,57 @@ void expire_tiles::from_point_list(geom::point_list_t const &list)
     });
 }
 
-void expire_tiles::from_geometry(geom::geometry_t const &geom, osmid_t osm_id)
+void expire_tiles::from_geometry(geom::point_t const &geom)
 {
-    if (geom.srid() != 3857) {
-        return;
-    }
+    geom::box_t const box = geom::envelope(geom);
+    from_bbox(box);
+}
 
-    if (geom.is_point()) {
-        auto const box = geom::envelope(geom);
-        from_bbox(box);
-    } else if (geom.is_linestring()) {
-        from_point_list(geom.get<geom::linestring_t>());
-    } else if (geom.is_multilinestring()) {
-        for (auto const &list : geom.get<geom::multilinestring_t>()) {
-            from_point_list(list);
-        }
-    } else if (geom.is_polygon() || geom.is_multipolygon()) {
-        auto const box = geom::envelope(geom);
-        if (from_bbox(box)) {
-            /* Bounding box too big - just expire tiles on the line */
-            log_debug("Large polygon ({:.0f} x {:.0f} metres, OSM ID {})"
-                      " - only expiring perimeter",
-                      box.max_x() - box.min_x(), box.max_y() - box.min_y(), osm_id);
-            if (geom.is_polygon()) {
-                from_point_list(geom.get<geom::polygon_t>().outer());
-                for (auto const &inner : geom.get<geom::polygon_t>().inners()) {
-                    from_point_list(inner);
-                }
-            } else if (geom.is_multipolygon()) {
-                for (auto const &polygon : geom.get<geom::multipolygon_t>()) {
-                    from_point_list(polygon.outer());
-                    for (auto const &inner : polygon.inners()) {
-                        from_point_list(inner);
-                    }
-                }
-            }
+void expire_tiles::from_geometry(geom::linestring_t const &geom)
+{
+    from_point_list(geom);
+}
+
+void expire_tiles::from_polygon_boundary(geom::polygon_t const &geom)
+{
+    from_point_list(geom.outer());
+    for (auto const &inner : geom.inners()) {
+        from_point_list(inner);
+    }
+}
+
+void expire_tiles::from_geometry(geom::polygon_t const &geom)
+{
+    geom::box_t const box = geom::envelope(geom);
+    if (from_bbox(box)) {
+        /* Bounding box too big - just expire tiles on the boundary */
+        from_polygon_boundary(geom);
+    }
+}
+
+void expire_tiles::from_geometry(geom::multipolygon_t const &geom)
+{
+    geom::box_t const box = geom::envelope(geom);
+    if (from_bbox(box)) {
+        /* Bounding box too big - just expire tiles on the boundary */
+        for (auto const &sgeom : geom) {
+            from_polygon_boundary(sgeom);
         }
     }
 }
 
+void expire_tiles::from_geometry(geom::geometry_t const &geom)
+{
+    geom.visit([&](auto const &g) { from_geometry(g); });
+}
+
+void expire_tiles::from_geometry_if_3857(geom::geometry_t const &geom)
+{
+    if (geom.srid() == 3857) {
+        from_geometry(geom);
+    }
+}
+
 /*
  * Expire tiles that a line crosses
  */
@@ -190,9 +173,7 @@ void expire_tiles::from_line(geom::point_t const &a, geom::point_t const &b)
            but for simplicity, treat the coordinates as a bounding box
            and expire everything within that box. */
         if (y1 > y2) {
-            double const temp = y2;
-            y2 = y1;
-            y1 = temp;
+            std::swap(y1, y2);
         }
         for (int x = x1 - tile_expiry_leeway; x <= x2 + tile_expiry_leeway;
              ++x) {
@@ -234,25 +215,17 @@ int expire_tiles::from_bbox(geom::box_t const &box)
 
     /* Convert the box's Mercator coordinates into tile coordinates */
     auto const tmp_min = coords_to_tile({box.min_x(), box.max_y()});
-    int min_tile_x = tmp_min.x() - tile_expiry_leeway;
-    int min_tile_y = tmp_min.y() - tile_expiry_leeway;
+    int const min_tile_x =
+        std::clamp(int(tmp_min.x() - tile_expiry_leeway), 0, m_map_width);
+    int const min_tile_y =
+        std::clamp(int(tmp_min.y() - tile_expiry_leeway), 0, m_map_width);
 
     auto const tmp_max = coords_to_tile({box.max_x(), box.min_y()});
-    int max_tile_x = tmp_max.x() + tile_expiry_leeway;
-    int max_tile_y = tmp_max.y() + tile_expiry_leeway;
+    int const max_tile_x =
+        std::clamp(int(tmp_max.x() + tile_expiry_leeway), 0, m_map_width);
+    int const max_tile_y =
+        std::clamp(int(tmp_max.y() + tile_expiry_leeway), 0, m_map_width);
 
-    if (min_tile_x < 0) {
-        min_tile_x = 0;
-    }
-    if (min_tile_y < 0) {
-        min_tile_y = 0;
-    }
-    if (max_tile_x > m_map_width) {
-        max_tile_x = m_map_width;
-    }
-    if (max_tile_y > m_map_width) {
-        max_tile_y = m_map_width;
-    }
     for (int iterator_x = min_tile_x; iterator_x <= max_tile_x; ++iterator_x) {
         uint32_t const norm_x = normalise_tile_x_coord(iterator_x);
         for (int iterator_y = min_tile_y; iterator_y <= max_tile_y;
@@ -263,34 +236,63 @@ int expire_tiles::from_bbox(geom::box_t const &box)
     return 0;
 }
 
-int expire_tiles::from_result(pg_result_t const &result, osmid_t osm_id)
+quadkey_list_t expire_tiles::get_tiles()
 {
-    if (!enabled()) {
-        return -1;
-    }
-
-    auto const num_tuples = result.num_tuples();
-    for (int i = 0; i < num_tuples; ++i) {
-        char const *const wkb = result.get_value(i, 0);
-        from_geometry(ewkb_to_geom(decode_hex(wkb)), osm_id);
-    }
-
-    return num_tuples;
+    quadkey_list_t tiles;
+    tiles.reserve(m_dirty_tiles.size());
+    tiles.assign(m_dirty_tiles.begin(), m_dirty_tiles.end());
+    std::sort(tiles.begin(), tiles.end());
+    m_dirty_tiles.clear();
+    return tiles;
 }
 
 void expire_tiles::merge_and_destroy(expire_tiles *other)
 {
     if (m_map_width != other->m_map_width) {
-        throw std::runtime_error{"Unable to merge tile expiry sets when "
-                                 "map_width does not match: {} != {}."_format(
-                                     m_map_width, other->m_map_width)};
+        throw fmt_error("Unable to merge tile expiry sets when "
+                        "map_width does not match: {} != {}.",
+                        m_map_width, other->m_map_width);
     }
 
     if (m_dirty_tiles.empty()) {
-        m_dirty_tiles = std::move(other->m_dirty_tiles);
+        using std::swap;
+        swap(m_dirty_tiles, other->m_dirty_tiles);
     } else {
         m_dirty_tiles.insert(other->m_dirty_tiles.begin(),
                              other->m_dirty_tiles.end());
         other->m_dirty_tiles.clear();
     }
 }
+
+std::size_t output_tiles_to_file(quadkey_list_t const &tiles_maxzoom,
+                                 char const *filename, uint32_t minzoom,
+                                 uint32_t maxzoom)
+{
+    FILE *outfile = std::fopen(filename, "a");
+    if (outfile == nullptr) {
+        log_warn("Failed to open expired tiles file ({}).  Tile expiry "
+                 "list will not be written!",
+                 std::strerror(errno));
+        return 0;
+    }
+
+    auto const count =
+        for_each_tile(tiles_maxzoom, minzoom, maxzoom, [&](tile_t const &tile) {
+            fmt::print(outfile, "{}/{}/{}\n", tile.zoom(), tile.x(), tile.y());
+        });
+
+    (void)std::fclose(outfile);
+
+    return count;
+}
+
+int expire_from_result(expire_tiles *expire, pg_result_t const &result)
+{
+    auto const num_tuples = result.num_tuples();
+
+    for (int i = 0; i < num_tuples; ++i) {
+        expire->from_geometry(ewkb_to_geom(result.get(i, 0)));
+    }
+
+    return num_tuples;
+}
