@@ -3,16 +3,18 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
 #include "flex-table.hpp"
 #include "format.hpp"
 #include "logging.hpp"
+#include "pgsql-capabilities.hpp"
 #include "pgsql-helper.hpp"
 #include "util.hpp"
 
+#include <algorithm>
 #include <cassert>
 #include <string>
 
@@ -63,6 +65,65 @@ std::string flex_table_t::full_tmp_name() const
     return qualified_name(schema(), name() + "_tmp");
 }
 
+bool flex_table_t::has_id_column() const noexcept
+{
+    if (m_columns.empty()) {
+        return false;
+    }
+    return (m_columns[0].type() == table_column_type::id_type) ||
+           (m_columns[0].type() == table_column_type::id_num);
+}
+
+bool flex_table_t::matches_type(osmium::item_type type) const noexcept
+{
+    // This table takes any type -> okay
+    if (m_id_type == osmium::item_type::undefined) {
+        return true;
+    }
+
+    // Type and table type match -> okay
+    if (type == m_id_type) {
+        return true;
+    }
+
+    // Relations can be written as linestrings into way tables -> okay
+    if (type == osmium::item_type::relation &&
+        m_id_type == osmium::item_type::way) {
+        return true;
+    }
+
+    // Area tables can take ways or relations, but not nodes
+    return m_id_type == osmium::item_type::area &&
+           type != osmium::item_type::node;
+}
+
+/// Map way/node/relation ID to id value used in database table column
+osmid_t flex_table_t::map_id(osmium::item_type type, osmid_t id) const noexcept
+{
+    if (m_id_type == osmium::item_type::undefined) {
+        if (has_multicolumn_id_index()) {
+            return id;
+        }
+
+        switch (type) {
+        case osmium::item_type::node:
+            return id;
+        case osmium::item_type::way:
+            return -id;
+        case osmium::item_type::relation:
+            return -id - 100000000000000000LL;
+        default:
+            assert(false);
+        }
+    }
+
+    if (m_id_type != osmium::item_type::relation &&
+        type == osmium::item_type::relation) {
+        return -id;
+    }
+    return id;
+}
+
 flex_table_column_t &flex_table_t::add_column(std::string const &name,
                                               std::string const &type,
                                               std::string const &sql_type)
@@ -89,15 +150,16 @@ flex_table_column_t &flex_table_t::add_column(std::string const &name,
 std::string flex_table_t::build_sql_prepare_get_wkb() const
 {
     if (has_multicolumn_id_index()) {
-        return "PREPARE get_wkb(char(1), bigint) AS"
-               " SELECT \"{}\" FROM {} WHERE \"{}\" = $1 AND \"{}\" = $2"_format(
-                   geom_column().name(), full_name(), m_columns[0].name(),
-                   m_columns[1].name());
+        return fmt::format(
+            "PREPARE get_wkb(char(1), bigint) AS"
+            " SELECT \"{}\" FROM {} WHERE \"{}\" = $1 AND \"{}\" = $2",
+            geom_column().name(), full_name(), m_columns[0].name(),
+            m_columns[1].name());
     }
 
-    return "PREPARE get_wkb(bigint) AS"
-           " SELECT \"{}\" FROM {} WHERE \"{}\" = $1"_format(
-               geom_column().name(), full_name(), id_column_names());
+    return fmt::format("PREPARE get_wkb(bigint) AS"
+                       " SELECT \"{}\" FROM {} WHERE \"{}\" = $1",
+                       geom_column().name(), full_name(), id_column_names());
 }
 
 std::string
@@ -106,19 +168,21 @@ flex_table_t::build_sql_create_table(table_type ttype,
 {
     assert(!m_columns.empty());
 
-    std::string sql = "CREATE {} TABLE IF NOT EXISTS {} ("_format(
-        ttype == table_type::interim ? "UNLOGGED" : "", table_name);
+    std::string sql =
+        fmt::format("CREATE {} TABLE IF NOT EXISTS {} (",
+                    ttype == table_type::interim ? "UNLOGGED" : "", table_name);
 
+    util::string_joiner_t joiner{','};
     for (auto const &column : m_columns) {
         // create_only columns are only created in permanent, not in the
         // interim tables
         if (ttype == table_type::permanent || !column.create_only()) {
-            sql += column.sql_create();
+            joiner.add(column.sql_create());
         }
     }
 
-    assert(sql.back() == ',');
-    sql.back() = ')';
+    sql += joiner();
+    sql += ')';
 
     if (ttype == table_type::interim) {
         sql += " WITH (autovacuum_enabled = off)";
@@ -133,24 +197,27 @@ std::string flex_table_t::build_sql_column_list() const
 {
     assert(!m_columns.empty());
 
-    std::string result;
+    util::string_joiner_t joiner{',', '"'};
+
     for (auto const &column : m_columns) {
         if (!column.create_only()) {
-            result += '"';
-            result += column.name();
-            result += '"';
-            result += ',';
+            joiner.add(column.name());
         }
     }
-    result.resize(result.size() - 1);
 
-    return result;
+    return joiner();
 }
 
 std::string flex_table_t::build_sql_create_id_index() const
 {
-    return "CREATE INDEX ON {} USING BTREE ({}) {}"_format(
-        full_name(), id_column_names(), tablespace_clause(index_tablespace()));
+    return fmt::format("CREATE INDEX ON {} USING BTREE ({}) {}", full_name(),
+                       id_column_names(),
+                       tablespace_clause(index_tablespace()));
+}
+
+flex_index_t &flex_table_t::add_index(std::string method)
+{
+    return m_indexes.emplace_back(std::move(method));
 }
 
 void table_connection_t::connect(std::string const &conninfo)
@@ -168,9 +235,9 @@ enable_check_trigger(pg_conn_t *db_connection, flex_table_t const &table)
 
     for (auto const &column : table) {
         if (column.is_geometry_column() && column.needs_isvalid()) {
-            checks.append(
-                R"((NEW."{0}" IS NULL OR ST_IsValid(NEW."{0}")) AND )"_format(
-                    column.name()));
+            checks.append(fmt::format(
+                R"((NEW."{0}" IS NULL OR ST_IsValid(NEW."{0}")) AND )",
+                column.name()));
         }
     }
 
@@ -192,13 +259,12 @@ void table_connection_t::start(bool append)
     m_db_connection->exec("SET client_min_messages = WARNING");
 
     if (!append) {
-        m_db_connection->exec(
-            "DROP TABLE IF EXISTS {} CASCADE"_format(table().full_name()));
+        m_db_connection->exec("DROP TABLE IF EXISTS {} CASCADE",
+                              table().full_name());
     }
 
     // These _tmp tables can be left behind if we run out of disk space.
-    m_db_connection->exec(
-        "DROP TABLE IF EXISTS {}"_format(table().full_tmp_name()));
+    m_db_connection->exec("DROP TABLE IF EXISTS {}", table().full_tmp_name());
     m_db_connection->exec("RESET client_min_messages");
 
     if (!append) {
@@ -240,21 +306,25 @@ void table_connection_t::stop(bool updateable, bool append)
             flex_table_t::table_type::permanent, table().full_tmp_name()));
 
         std::string const columns = table().build_sql_column_list();
-        std::string sql = "INSERT INTO {} ({}) SELECT {} FROM {}"_format(
-            table().full_tmp_name(), columns, columns, table().full_name());
+        std::string sql = fmt::format("INSERT INTO {} ({}) SELECT {} FROM {}",
+                                      table().full_tmp_name(), columns, columns,
+                                      table().full_name());
 
-        auto const postgis_version = get_postgis_version(*m_db_connection);
+        auto const postgis_version = get_postgis_version();
+
+        auto const geom_column_name =
+            "\"" + table().geom_column().name() + "\"";
 
         sql += " ORDER BY ";
         if (postgis_version.major == 2 && postgis_version.minor < 4) {
             log_debug("Using GeoHash for clustering table '{}'",
                       table().name());
             if (table().geom_column().srid() == 4326) {
-                sql += "ST_GeoHash({},10)"_format(table().geom_column().name());
+                sql += fmt::format("ST_GeoHash({},10)", geom_column_name);
             } else {
-                sql +=
-                    "ST_GeoHash(ST_Transform(ST_Envelope({}),4326),10)"_format(
-                        table().geom_column().name());
+                sql += fmt::format(
+                    "ST_GeoHash(ST_Transform(ST_Envelope({}),4326),10)",
+                    geom_column_name);
             }
             sql += " COLLATE \"C\"";
         } else {
@@ -262,14 +332,14 @@ void table_connection_t::stop(bool updateable, bool append)
                       table().name());
             // Since Postgis 2.4 the order function for geometries gives
             // useful results.
-            sql += table().geom_column().name();
+            sql += geom_column_name;
         }
 
         m_db_connection->exec(sql);
 
-        m_db_connection->exec("DROP TABLE {}"_format(table().full_name()));
-        m_db_connection->exec(R"(ALTER TABLE {} RENAME TO "{}")"_format(
-            table().full_tmp_name(), table().name()));
+        m_db_connection->exec("DROP TABLE {}", table().full_name());
+        m_db_connection->exec(R"(ALTER TABLE {} RENAME TO "{}")",
+                              table().full_tmp_name(), table().name());
         m_id_index_created = false;
 
         if (updateable) {
@@ -277,18 +347,20 @@ void table_connection_t::stop(bool updateable, bool append)
         }
     }
 
-    if (table().has_geom_column()) {
-        log_info("Creating geometry index on table '{}'...", table().name());
-
-        // Use fillfactor 100 for un-updateable imports
-        m_db_connection->exec(
-            R"(CREATE INDEX ON {} USING GIST ("{}") {} {})"_format(
-                table().full_name(), table().geom_column().name(),
-                (updateable ? "" : "WITH (fillfactor = 100)"),
-                tablespace_clause(table().index_tablespace())));
+    if (table().indexes().empty()) {
+        log_info("No indexes to create on table '{}'.", table().name());
+    } else {
+        for (auto const &index : table().indexes()) {
+            log_info("Creating index on table '{}' {}...", table().name(),
+                     index.columns());
+            auto const sql = index.create_index(
+                qualified_name(table().schema(), table().name()));
+            m_db_connection->exec(sql);
+        }
     }
 
-    if (updateable && table().has_id_column()) {
+    if (table().always_build_id_index() ||
+        (updateable && table().has_id_column())) {
         create_id_index();
     }
 
@@ -327,12 +399,11 @@ pg_result_t table_connection_t::get_geom_by_id(osmium::item_type type,
 {
     assert(table().has_geom_column());
     assert(m_db_connection);
-    std::string const id_str = fmt::to_string(id);
     if (table().has_multicolumn_id_index()) {
-        return m_db_connection->exec_prepared(
-            "get_wkb", type_to_char(type), id_str.c_str());
+        return m_db_connection->exec_prepared_as_binary("get_wkb",
+                                                        type_to_char(type), id);
     }
-    return m_db_connection->exec_prepared("get_wkb", id_str);
+    return m_db_connection->exec_prepared_as_binary("get_wkb", id);
 }
 
 void table_connection_t::delete_rows_with(osmium::item_type type, osmid_t id)
