@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -55,7 +55,9 @@ public:
         str += fmt::format(ts, format_str, std::forward<TArgs>(args)...);
         str += '\n';
 
-        std::fputs(str.c_str(), stderr);
+        if (std::fputs(str.c_str(), stderr) < 0) {
+            throw std::runtime_error{"Can not write to log"};
+        }
     }
 
     bool log_sql() const noexcept { return m_log_sql; }
