@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -18,6 +18,7 @@ extern "C"
 #include <lua.h>
 }
 
+#include <cassert>
 #include <cstdint>
 #include <utility>
 
@@ -64,4 +65,47 @@ bool luaX_get_table_bool(lua_State *lua_state, char const *key, int table_index,
 
 int luaX_pcall(lua_State *lua_state, int narg, int nres);
 
+/**
+ * Returns true if the value on top of the stack is an empty Lua table.
+ *
+ * \pre Value on top of the Lua stack must be a Lua table.
+ * \post Stack is unchanged.
+ */
+bool luaX_is_empty_table(lua_State *lua_state);
+
+/**
+ * Check that the value on the top of the Lua stack is a simple array.
+ * This means that all keys must be consecutive integers starting from 1.
+ *
+ * \returns True if this is an array (also for Lua tables without any items)
+ *
+ * \pre Value on top of the Lua stack must be a Lua table.
+ * \post Stack is unchanged.
+ */
+bool luaX_is_array(lua_State *lua_state);
+
+/**
+ * Call a function for each item in a Lua array table. The item value will
+ * be on the top of the stack inside that function.
+ *
+ * \pre Value on top of the Lua stack must be a Lua array table.
+ * \pre The function must leave the Lua stack in the same condition it found
+ *      it in.
+ * \post Stack is unchanged.
+ */
+template <typename FUNC>
+void luaX_for_each(lua_State *lua_state, FUNC &&func)
+{
+    assert(lua_istable(lua_state, -1));
+    lua_pushnil(lua_state);
+    while (lua_next(lua_state, -2) != 0) {
+#ifndef NDEBUG
+        int const top = lua_gettop(lua_state);
+#endif
+        std::forward<FUNC>(func)();
+        assert(top == lua_gettop(lua_state));
+        lua_pop(lua_state, 1);
+    }
+}
+
 #endif // OSM2PGSQL_LUA_UTILS_HPP
