@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -85,7 +85,7 @@ void middle_pgsql_t::table_desc::drop_table(
     log_info("Dropping table '{}'", name());
 
     auto const qual_name = qualified_name(schema(), name());
-    db_connection.exec("DROP TABLE IF EXISTS {}"_format(qual_name));
+    db_connection.exec("DROP TABLE IF EXISTS {}", qual_name);
 
     log_info("Table '{}' dropped in {}", name(),
              util::human_readable_duration(timer.stop()));
@@ -99,47 +99,62 @@ void middle_pgsql_t::table_desc::build_index(std::string const &conninfo) const
 
     // Use a temporary connection here because we might run in a separate
     // thread context.
-    pg_conn_t db_connection{conninfo};
+    pg_conn_t const db_connection{conninfo};
 
     log_info("Building index on table '{}'", name());
     db_connection.exec(m_create_fw_dep_indexes);
 }
 
-namespace {
-// Decodes a portion of an array literal from postgres */
-// Argument should point to beginning of literal, on return points to delimiter */
-inline char const *decode_upto(char const *src, char *dst)
+/**
+ * Decode item in an array literal from PostgreSQL to the next delimiter.
+ *
+ * \param src Pointer to the text with the array literal.
+ * \param dst The decoded item is written to this string. The string is
+ *            cleared before use.
+ * \returns Pointer to the delimiter found.
+ * \throws runtime_error If the input string ends before the delimiter is found.
+ */
+static char const *decode_to_delimiter(char const *src, std::string *dst)
 {
+    assert(src);
+    dst->clear();
+
     bool const quoted = (*src == '"');
     if (quoted) {
         ++src;
     }
 
     while (quoted ? (*src != '"') : (*src != ',' && *src != '}')) {
+        if (*src == '\0') {
+            throw std::runtime_error{
+                "Parsing array literal from database failed."};
+        }
         if (*src == '\\') {
             switch (src[1]) {
             case 'n':
-                *dst++ = '\n';
+                dst->append(1, '\n');
                 break;
             case 't':
-                *dst++ = '\t';
+                dst->append(1, '\t');
                 break;
             default:
-                *dst++ = src[1];
+                dst->append(1, src[1]);
                 break;
             }
             src += 2;
         } else {
-            *dst++ = *src++;
+            dst->append(1, *src++);
         }
     }
+
     if (quoted) {
         ++src;
     }
-    *dst = 0;
+
     return src;
 }
 
+namespace {
 template <typename T>
 void pgsql_parse_tags(char const *string, osmium::memory::Buffer *buffer,
                       T *obuilder)
@@ -148,15 +163,15 @@ void pgsql_parse_tags(char const *string, osmium::memory::Buffer *buffer,
         return;
     }
 
-    char key[1024];
-    char val[1024];
     osmium::builder::TagListBuilder builder{*buffer, obuilder};
 
+    std::string key;
+    std::string val;
     while (*string != '}') {
-        string = decode_upto(string, key);
+        string = decode_to_delimiter(string, &key);
         // String points to the comma */
         ++string;
-        string = decode_upto(string, val);
+        string = decode_to_delimiter(string, &val);
         builder.add_tag(key, val);
         // String points to the comma or closing '}' */
         if (*string == ',') {
@@ -172,15 +187,15 @@ void pgsql_parse_members(char const *string, osmium::memory::Buffer *buffer,
         return;
     }
 
-    char role[1024];
+    std::string role;
     osmium::builder::RelationMemberListBuilder builder{*buffer, obuilder};
 
     while (*string != '}') {
-        char type = string[0];
+        char const type = string[0];
         char *endp = nullptr;
-        osmid_t id = std::strtoll(string + 1, &endp, 10);
+        osmid_t const id = std::strtoll(string + 1, &endp, 10);
         // String points to the comma */
-        string = decode_upto(endp + 1, role);
+        string = decode_to_delimiter(endp + 1, &role);
         builder.add_member(osmium::char_to_item_type(type), id, role);
         // String points to the comma or closing '}' */
         if (*string == ',') {
@@ -236,7 +251,7 @@ std::size_t middle_query_pgsql_t::get_way_node_locations_db(
     osmium::WayNodeList *nodes) const
 {
     size_t count = 0;
-    util::string_id_list_t id_list;
+    util::string_joiner_t id_list{',', '\0', '{', '}'};
 
     // get nodes where possible from cache,
     // at the same time build a list for querying missing nodes from DB
@@ -246,7 +261,7 @@ std::size_t middle_query_pgsql_t::get_way_node_locations_db(
             n.set_location(loc);
             ++count;
         } else {
-            id_list.add(n.ref());
+            id_list.add(fmt::to_string(n.ref()));
         }
     }
 
@@ -256,7 +271,7 @@ std::size_t middle_query_pgsql_t::get_way_node_locations_db(
 
     // get any remaining nodes from the DB
     // Nodes must have been written back at this point.
-    auto const res = m_sql_conn.exec_prepared("get_node_list", id_list.get());
+    auto const res = m_sql_conn.exec_prepared("get_node_list", id_list());
     std::unordered_map<osmid_t, osmium::Location> locs;
     for (int i = 0; i < res.num_tuples(); ++i) {
         locs.emplace(
@@ -347,8 +362,7 @@ std::size_t middle_query_pgsql_t::get_way_node_locations_flatnodes(
 
 osmium::Location middle_query_pgsql_t::get_node_location_db(osmid_t id) const
 {
-    auto const res =
-        m_sql_conn.exec_prepared("get_node_list", "{{{}}}"_format(id));
+    auto const res = m_sql_conn.exec_prepared("get_node", id);
     if (res.num_tuples() == 0) {
         return osmium::Location{};
     }
@@ -460,42 +474,39 @@ middle_query_pgsql_t::rel_members_get(osmium::Relation const &rel,
     assert(buffer);
     assert((types & osmium::osm_entity_bits::relation) == 0);
 
-    util::string_id_list_t way_ids;
-    idlist_t node_ids;
+    pg_result_t res;
+    idlist_t wayidspg;
+    if (types & osmium::osm_entity_bits::way) {
+        // collect ids from all way members into a list..
+        util::string_joiner_t way_ids{',', '\0', '{', '}'};
+        for (auto const &member : rel.members()) {
+            if (member.type() == osmium::item_type::way) {
+                way_ids.add(fmt::to_string(member.ref()));
+            }
+        }
 
+        // ...and get those ways from database
+        if (!way_ids.empty()) {
+            res = m_sql_conn.exec_prepared("get_way_list", way_ids());
+            wayidspg = get_ids_from_result(res);
+        }
+    }
+
+    std::size_t members_found = 0;
     for (auto const &member : rel.members()) {
         if (member.type() == osmium::item_type::node &&
             (types & osmium::osm_entity_bits::node)) {
-            node_ids.push_back(member.ref());
+            osmium::builder::NodeBuilder builder{*buffer};
+            builder.set_id(member.ref());
+            ++members_found;
         } else if (member.type() == osmium::item_type::way &&
-                   (types & osmium::osm_entity_bits::way)) {
-            way_ids.add(member.ref());
-        }
-    }
-
-    std::size_t members_found = node_ids.size();
-    if (!node_ids.empty()) {
-        osmium::builder::WayNodeListBuilder builder{*buffer};
-        for (auto const id : node_ids) {
-            builder.add_node_ref(id);
-        }
-    }
-
-    if (!way_ids.empty()) {
-        auto const res =
-            m_sql_conn.exec_prepared("get_way_list", way_ids.get());
-        idlist_t const wayidspg = get_ids_from_result(res);
-
-        // Match the list of ways coming from postgres in a different order
-        //   back to the list of ways given by the caller
-        for (auto const &m : rel.members()) {
-            if (m.type() != osmium::item_type::way) {
-                continue;
-            }
+                   (types & osmium::osm_entity_bits::way) && res) {
+            // Match the list of ways coming from postgres in a different order
+            // back to the list of ways given by the caller
             for (int j = 0; j < res.num_tuples(); ++j) {
-                if (m.ref() == wayidspg[static_cast<std::size_t>(j)]) {
+                if (member.ref() == wayidspg[static_cast<std::size_t>(j)]) {
                     osmium::builder::WayBuilder builder{*buffer};
-                    builder.set_id(m.ref());
+                    builder.set_id(member.ref());
 
                     pgsql_parse_nodes(res.get_value(j, 1), buffer, &builder);
                     pgsql_parse_tags(res.get_value(j, 2), buffer, &builder);
@@ -656,9 +667,10 @@ void middle_pgsql_t::start()
         for (auto const &table : m_tables) {
             log_debug("Setting up table '{}'", table.name());
             auto const qual_name = qualified_name(table.schema(), table.name());
-            m_db_connection.exec(
-                "DROP TABLE IF EXISTS {} CASCADE"_format(qual_name));
-            m_db_connection.exec(table.m_create_table);
+            m_db_connection.exec("DROP TABLE IF EXISTS {} CASCADE", qual_name);
+            if (!table.m_create_table.empty()) {
+                m_db_connection.exec(table.m_create_table);
+            }
         }
     }
 }
@@ -679,9 +691,8 @@ void middle_pgsql_t::stop()
     } else if (!m_options->append) {
         // Building the indexes takes time, so do it asynchronously.
         for (auto &table : m_tables) {
-            table.task_set(thread_pool().submit([&]() {
-                table.build_index(m_options->database_options.conninfo());
-            }));
+            table.task_set(thread_pool().submit(
+                [&]() { table.build_index(m_options->conninfo); }));
         }
     }
 }
@@ -712,7 +723,10 @@ static table_sql sql_for_nodes(bool create_table) noexcept
         sql.prepare_query =
             "PREPARE get_node_list(int8[]) AS"
             "  SELECT id, lon, lat FROM {schema}\"{prefix}_nodes\""
-            "  WHERE id = ANY($1::int8[]);\n";
+            "  WHERE id = ANY($1::int8[]);\n"
+            "PREPARE get_node(int8) AS"
+            "  SELECT id, lon, lat FROM {schema}\"{prefix}_nodes\""
+            "  WHERE id = $1;\n";
     }
 
     return sql;
@@ -813,10 +827,11 @@ static table_sql sql_for_relations() noexcept
 static bool check_bucket_index(pg_conn_t *db_connection,
                                std::string const &prefix)
 {
-    auto const res = db_connection->query(
-        PGRES_TUPLES_OK,
-        "SELECT relname FROM pg_class WHERE relkind='i' AND"
-        "  relname = '{}_ways_nodes_bucket_idx';"_format(prefix));
+    auto const res =
+        db_connection->exec("SELECT relname FROM pg_class"
+                            " WHERE relkind='i'"
+                            " AND relname = '{}_ways_nodes_bucket_idx'",
+                            prefix);
     return res.num_tuples() > 0;
 }
 
@@ -825,9 +840,8 @@ middle_pgsql_t::middle_pgsql_t(std::shared_ptr<thread_pool_t> thread_pool,
 : middle_t(std::move(thread_pool)), m_options(options),
   m_cache(std::make_unique<node_locations_t>(
       static_cast<std::size_t>(options->cache) * 1024UL * 1024UL)),
-  m_db_connection(m_options->database_options.conninfo()),
-  m_copy_thread(
-      std::make_shared<db_copy_thread_t>(options->database_options.conninfo())),
+  m_db_connection(m_options->conninfo),
+  m_copy_thread(std::make_shared<db_copy_thread_t>(options->conninfo)),
   m_db_copy(m_copy_thread)
 {
     if (!options->flat_node_file.empty()) {
@@ -859,11 +873,13 @@ middle_pgsql_t::get_query_instance()
     // NOTE: this is thread safe for use in pending async processing only because
     // during that process they are only read from
     auto mid = std::make_unique<middle_query_pgsql_t>(
-        m_options->database_options.conninfo(), m_cache, m_persistent_cache);
+        m_options->conninfo, m_cache, m_persistent_cache);
 
     // We use a connection per table to enable the use of COPY
     for (auto &table : m_tables) {
-        mid->exec_sql(table.m_prepare_query);
+        if (!table.m_prepare_query.empty()) {
+            mid->exec_sql(table.m_prepare_query);
+        }
     }
 
     return std::shared_ptr<middle_query_t>(mid.release());
