@@ -3,17 +3,17 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
-#include "config.h"
 #include "format.hpp"
 #include "logging.hpp"
-#include "node-ram-cache.hpp"
 #include "options.hpp"
+#include "pgsql.hpp"
 #include "reprojection.hpp"
-#include "sprompt.hpp"
+#include "util.hpp"
+#include "version.hpp"
 
 #include <algorithm>
 #include <cstdio>
@@ -44,8 +44,10 @@ static char const *program_name(char const *name)
 }
 
 namespace {
-char const *short_options = "ab:cd:KhlmMp:suvU:WH:P:i:IE:C:S:e:o:O:xkjGz:r:VF:";
-const struct option long_options[] = {
+char const *const short_options =
+    "ab:cd:KhlmMp:suvU:WH:P:i:IE:C:S:e:o:O:xkjGz:r:VF:";
+
+struct option const long_options[] = {
     {"append", no_argument, nullptr, 'a'},
     {"bbox", required_argument, nullptr, 'b'},
     {"cache", required_argument, nullptr, 'C'},
@@ -99,17 +101,17 @@ const struct option long_options[] = {
     {"with-forward-dependencies", required_argument, nullptr, 217},
     {nullptr, 0, nullptr, 0}};
 
-static void long_usage(char const *arg0, bool verbose)
+void long_usage(char const *arg0, bool verbose)
 {
     char const *const name = program_name(arg0);
 
     fmt::print(stdout, "\nUsage: {} [OPTIONS] OSM-FILE...\n", name);
-    std::fputs(
+    (void)std::fputs(
         "\nImport data from the OSM file(s) into a PostgreSQL database.\n\n\
 Full documentation is available at https://osm2pgsql.org/\n\n",
         stdout);
 
-    std::fputs("\
+    (void)std::fputs("\
 Common options:\n\
     -a|--append     Update existing osm2pgsql database with data from file.\n\
     -c|--create     Import OSM data from file into database. This is the\n\
@@ -117,21 +119,20 @@ Common options:\n\
     -O|--output=OUTPUT  Set output. Options are:\n\
                     pgsql - Output to a PostGIS database (default)\n\
                     flex - More flexible output to PostGIS database\n\
-                    multi - Multiple Custom Table Output to a PostGIS \n\
-                            database (deprecated)\n\
                     gazetteer - Output to a PostGIS database for Nominatim\n\
+                                (deprecated)\n\
                     null - No output. Used for testing.\n\
     -S|--style=FILE  Location of the style file. Defaults to\n\
                     '" DEFAULT_STYLE "'.\n\
     -k|--hstore     Add tags without column to an additional hstore column.\n",
                stdout);
 #ifdef HAVE_LUA
-    std::fputs("\
+    (void)std::fputs("\
        --tag-transform-script=SCRIPT  Specify a Lua script to handle tag\n\
                     filtering and normalisation (pgsql output only).\n",
                stdout);
 #endif
-    std::fputs("\
+    (void)std::fputs("\
     -s|--slim       Store temporary data in the database. This switch is\n\
                     required if you want to update with --append later.\n\
         --drop      Only with --slim: drop temporary tables after import\n\
@@ -152,7 +153,7 @@ Database options:\n\
                stdout);
 
     if (verbose) {
-        std::fputs("\n\
+        (void)std::fputs("\n\
 Logging options:\n\
        --log-level=LEVEL  Set log level ('debug', 'info' (default), 'warn',\n\
                     or 'error').\n\
@@ -179,22 +180,7 @@ Middle options:\n\
                     (if unset, use db's default; -i is equivalent to setting\n\
                     --tablespace-main-index and --tablespace-slim-index).\n\
     -p|--prefix=PREFIX  Prefix for table names (default 'planet_osm')\n\
-       --cache-strategy=STRATEGY  Specifies the method used to cache nodes in\n\
-                    RAM. Available options are:\n\
-                    dense: caching strategy optimised for full planet import\n\
-                    chunk: caching strategy optimised for non-contiguous\n\
-                           memory allocation\n\
-                    sparse: caching strategy optimised for small imports\n\
-                    optimized: automatically combines dense and sparse \n\
-                           strategies for optimal storage efficiency. This may\n\
-                           us twice as much virtual memory, but no more physical\n\
-                           memory.\n"
-#ifdef __amd64__
-                   "                    The default is 'optimized'.\n"
-#else
-                   "                    The default is 'sparse'.\n"
-#endif
-                   "\
+       --cache-strategy=STRATEGY  Deprecated. Not used any more.\n\
     -x|--extra-attributes  Include attributes (user name, user id, changeset\n\
                     id, timestamp and version) for each object in the database.\n\
        --middle-schema=SCHEMA  Schema to use for middle tables (default: none).\n\
@@ -226,7 +212,7 @@ Pgsql output options:\n\
     -G|--multi-geometry  Generate multi-geometry features in postgresql tables.\n\
     -K|--keep-coastlines  Keep coastline data rather than filtering it out.\n\
                     Default: discard objects tagged natural=coastline.\n\
-       --output-pgsql-schema=SCHEMA Schema to use for pgsql/multi output tables\n\
+       --output-pgsql-schema=SCHEMA Schema to use for pgsql output tables\n\
                     (default: none).\n\
        --reproject-area  Compute area column using web mercator coordinates.\n\
 \n\
@@ -261,48 +247,44 @@ static bool compare_prefix(std::string const &str,
     return std::strncmp(str.c_str(), prefix.c_str(), prefix.size()) == 0;
 }
 
-std::string database_options_t::conninfo() const
+std::string build_conninfo(database_options_t const &opt)
 {
-    if (compare_prefix(db, "postgresql://") ||
-        compare_prefix(db, "postgres://")) {
-        return db;
+    if (compare_prefix(opt.db, "postgresql://") ||
+        compare_prefix(opt.db, "postgres://")) {
+        return opt.db;
     }
 
-    std::string out{"fallback_application_name='osm2pgsql'"};
+    util::string_joiner_t joiner{' '};
+    joiner.add("fallback_application_name='osm2pgsql'");
 
-    if (std::strchr(db.c_str(), '=') != nullptr) {
-        out += " ";
-        out += db;
-        return out;
+    if (std::strchr(opt.db.c_str(), '=') != nullptr) {
+        joiner.add(opt.db);
+        return joiner();
     }
 
-    if (!db.empty()) {
-        out += " dbname='{}'"_format(db);
+    joiner.add("client_encoding='UTF8'");
+
+    if (!opt.db.empty()) {
+        joiner.add(fmt::format("dbname='{}'", opt.db));
     }
-    if (!username.empty()) {
-        out += " user='{}'"_format(username);
+    if (!opt.username.empty()) {
+        joiner.add(fmt::format("user='{}'", opt.username));
     }
-    if (!password.empty()) {
-        out += " password='{}'"_format(password);
+    if (!opt.password.empty()) {
+        joiner.add(fmt::format("password='{}'", opt.password));
     }
-    if (!host.empty()) {
-        out += " host='{}'"_format(host);
+    if (!opt.host.empty()) {
+        joiner.add(fmt::format("host='{}'", opt.host));
     }
-    if (!port.empty()) {
-        out += " port='{}'"_format(port);
+    if (!opt.port.empty()) {
+        joiner.add(fmt::format("port='{}'", opt.port));
     }
 
-    return out;
+    return joiner();
 }
 
 options_t::options_t()
-:
-#ifdef __amd64__
-  alloc_chunkwise(ALLOC_SPARSE | ALLOC_DENSE),
-#else
-  alloc_chunkwise(ALLOC_SPARSE),
-#endif
-  num_procs((int)std::min(4U, std::thread::hardware_concurrency()))
+: num_procs(std::min(4U, std::thread::hardware_concurrency()))
 {
     if (num_procs < 1) {
         log_warn("Unable to detect number of hardware threads supported!"
@@ -311,14 +293,14 @@ options_t::options_t()
     }
 }
 
-static osmium::Box parse_bbox(char const *bbox)
+static osmium::Box parse_bbox_param(char const *arg)
 {
-    double minx;
-    double maxx;
-    double miny;
-    double maxy;
+    double minx = NAN;
+    double maxx = NAN;
+    double miny = NAN;
+    double maxy = NAN;
 
-    int const n = sscanf(bbox, "%lf,%lf,%lf,%lf", &minx, &miny, &maxx, &maxy);
+    int const n = sscanf(arg, "%lf,%lf,%lf,%lf", &minx, &miny, &maxx, &maxy);
     if (n != 4) {
         throw std::runtime_error{"Bounding box must be specified like: "
                                  "minlon,minlat,maxlon,maxlat."};
@@ -339,6 +321,136 @@ static osmium::Box parse_bbox(char const *bbox)
     return osmium::Box{minx, miny, maxx, maxy};
 }
 
+static unsigned int parse_number_processes_param(char const *arg)
+{
+    int num = atoi(arg);
+    if (num < 1) {
+        log_warn("--number-processes must be at least 1. Using 1.");
+        num = 1;
+    } else if (num > 32) {
+        // The threads will open up database connections which will
+        // run out at some point. It depends on the number of tables
+        // how many connections there are. The number 32 is way beyond
+        // anything that will make sense here.
+        log_warn("--number-processes too large. Set to 32.");
+        num = 32;
+    }
+
+    return static_cast<unsigned int>(num);
+}
+
+static void parse_expire_tiles_param(char const *arg,
+                                     uint32_t *expire_tiles_zoom_min,
+                                     uint32_t *expire_tiles_zoom)
+{
+    if (!arg || arg[0] == '-') {
+        throw std::runtime_error{"Missing argument for option --expire-tiles."
+                                 " Zoom levels must be positive."};
+    }
+
+    char *next_char = nullptr;
+    *expire_tiles_zoom_min =
+        static_cast<uint32_t>(std::strtoul(arg, &next_char, 10));
+
+    if (*expire_tiles_zoom_min == 0) {
+        throw std::runtime_error{"Bad argument for option --expire-tiles."
+                                 " Minimum zoom level must be larger than 0."};
+    }
+
+    // The first character after the number is ignored because that is the
+    // separating hyphen.
+    if (*next_char == '-') {
+        ++next_char;
+        // Second number must not be negative because zoom levels must be
+        // positive.
+        if (next_char && *next_char != '-' && isdigit(*next_char)) {
+            char *after_maxzoom = nullptr;
+            *expire_tiles_zoom = static_cast<uint32_t>(
+                std::strtoul(next_char, &after_maxzoom, 10));
+            if (*expire_tiles_zoom == 0 || *after_maxzoom != '\0') {
+                throw std::runtime_error{"Invalid maximum zoom level"
+                                         " given for tile expiry."};
+            }
+        } else {
+            throw std::runtime_error{
+                "Invalid maximum zoom level given for tile expiry."};
+        }
+        return;
+    }
+
+    if (*next_char == '\0') {
+        // end of string, no second zoom level given
+        *expire_tiles_zoom = *expire_tiles_zoom_min;
+        return;
+    }
+
+    throw std::runtime_error{"Minimum and maximum zoom level for"
+                             " tile expiry must be separated by '-'."};
+}
+
+static void parse_log_level_param(char const *arg)
+{
+    if (std::strcmp(arg, "debug") == 0) {
+        get_logger().set_level(log_level::debug);
+    } else if (std::strcmp(arg, "info") == 0) {
+        get_logger().set_level(log_level::info);
+    } else if ((std::strcmp(arg, "warn") == 0) ||
+               (std::strcmp(arg, "warning") == 0)) {
+        get_logger().set_level(log_level::warn);
+    } else if (std::strcmp(arg, "error") == 0) {
+        get_logger().set_level(log_level::error);
+    } else {
+        throw fmt_error("Unknown value for --log-level option: {}", arg);
+    }
+}
+
+static void parse_log_progress_param(char const *arg)
+{
+    if (std::strcmp(arg, "true") == 0) {
+        get_logger().enable_progress();
+    } else if (std::strcmp(arg, "false") == 0) {
+        get_logger().disable_progress();
+    } else if (std::strcmp(arg, "auto") == 0) {
+        get_logger().auto_progress();
+    } else {
+        throw fmt_error("Unknown value for --log-progress option: {}", arg);
+    }
+}
+
+static bool parse_with_forward_dependencies_param(char const *arg)
+{
+    log_warn("The option --with-forward-dependencies is deprecated and will "
+             "soon be removed.");
+
+    if (std::strcmp(arg, "false") == 0) {
+        return false;
+    }
+
+    if (std::strcmp(arg, "true") == 0) {
+        return true;
+    }
+
+    throw fmt_error("Unknown value for --with-forward-dependencies option: {}",
+                    arg);
+}
+
+static void print_version()
+{
+    fmt::print(stderr, "Build: {}\n", get_build_type());
+    fmt::print(stderr, "Compiled using the following library versions:\n");
+    fmt::print(stderr, "Libosmium {}\n", LIBOSMIUM_VERSION_STRING);
+    fmt::print(stderr, "Proj {}\n", get_proj_version());
+#ifdef HAVE_LUA
+#ifdef HAVE_LUAJIT
+    fmt::print(stderr, "{} ({})\n", LUA_RELEASE, LUAJIT_VERSION);
+#else
+    fmt::print(stderr, "{}\n", LUA_RELEASE);
+#endif
+#else
+    fmt::print(stderr, "Lua support not included\n");
+#endif
+}
+
 options_t::options_t(int argc, char *argv[]) : options_t()
 {
     // If there are no command line arguments at all, show help.
@@ -348,287 +460,190 @@ options_t::options_t(int argc, char *argv[]) : options_t()
         return;
     }
 
+    database_options_t database_options;
     bool help_verbose = false; // Will be set when -v/--verbose is set
 
-    int c;
+    int c = 0;
 
     //keep going while there are args left to handle
     // note: optind would seem to need to be set to 1, but that gives valgrind
     // errors - setting it to zero seems to work, though. see
     // http://stackoverflow.com/questions/15179963/is-it-possible-to-repeat-getopt#15179990
     optind = 0;
+    // NOLINTNEXTLINE(concurrency-mt-unsafe)
     while (-1 != (c = getopt_long(argc, argv, short_options, long_options,
                                   nullptr))) {
 
         //handle the current arg
         switch (c) {
-        case 'a':
+        case 'a': // --append
             append = true;
             break;
-        case 'b':
-            bbox = parse_bbox(optarg);
+        case 'b': // --bbox
+            bbox = parse_bbox_param(optarg);
             break;
-        case 'c':
+        case 'c': // --create
             create = true;
             break;
-        case 'v':
+        case 'v': // --verbose
             help_verbose = true;
             get_logger().set_level(log_level::debug);
             break;
-        case 's':
+        case 's': // --slim
             slim = true;
             break;
-        case 'K':
+        case 'K': // --keep-coastlines
             keep_coastlines = true;
             break;
-        case 'l':
+        case 'l': // --latlong
             projection = reprojection::create_projection(PROJ_LATLONG);
             break;
-        case 'm':
+        case 'm': // --merc
             projection = reprojection::create_projection(PROJ_SPHERE_MERC);
             break;
-        case 'E':
+        case 'E': // --proj
 #ifdef HAVE_GENERIC_PROJ
             projection = reprojection::create_projection(atoi(optarg));
 #else
             throw std::runtime_error{"Generic projections not available."};
 #endif
             break;
-        case 'p':
+        case 'p': // --prefix
             prefix = optarg;
+            check_identifier(prefix, "--prefix parameter");
             break;
-        case 'd':
+        case 'd': // --database
             database_options.db = optarg;
             break;
-        case 'C':
+        case 'C': // --cache
             cache = atoi(optarg);
             break;
-        case 'U':
+        case 'U': // --username
             database_options.username = optarg;
             break;
-        case 'W':
+        case 'W': // --password
             pass_prompt = true;
             break;
-        case 'H':
+        case 'H': // --host
             database_options.host = optarg;
             break;
-        case 'P':
+        case 'P': // --port
             database_options.port = optarg;
             break;
-        case 'S':
+        case 'S': // --style
             style = optarg;
             break;
-        case 'i':
+        case 'i': // --tablespace-index
             tblsmain_index = optarg;
             tblsslim_index = tblsmain_index;
             break;
-        case 200:
+        case 200: // --tablespace-slim-data
             tblsslim_data = optarg;
             break;
-        case 201:
+        case 201: // --tablespace-slim-index
             tblsslim_index = optarg;
             break;
-        case 202:
+        case 202: // --tablespace-main-data
             tblsmain_data = optarg;
             break;
-        case 203:
+        case 203: // --tablespace-main-index
             tblsmain_index = optarg;
             break;
-        case 'e':
-            if (!optarg || optarg[0] == '-') {
-                throw std::runtime_error{
-                    "Missing argument for option --expire-tiles. Zoom "
-                    "levels must be positive."};
-            }
-            char *next_char;
-            expire_tiles_zoom_min =
-                static_cast<uint32_t>(std::strtoul(optarg, &next_char, 10));
-            if (expire_tiles_zoom_min == 0) {
-                throw std::runtime_error{
-                    "Bad argument for option --expire-tiles. "
-                    "Minimum zoom level must be larger "
-                    "than 0."};
-            }
-            // The first character after the number is ignored because that is the separating hyphen.
-            if (*next_char == '-') {
-                ++next_char;
-                // Second number must not be negative because zoom levels must be positive.
-                if (next_char && *next_char != '-' && isdigit(*next_char)) {
-                    char *after_maxzoom;
-                    expire_tiles_zoom = static_cast<uint32_t>(
-                        std::strtoul(next_char, &after_maxzoom, 10));
-                    if (expire_tiles_zoom == 0 || *after_maxzoom != '\0') {
-                        throw std::runtime_error{"Invalid maximum zoom level "
-                                                 "given for tile expiry."};
-                    }
-                } else {
-                    throw std::runtime_error{
-                        "Invalid maximum zoom level given for tile expiry."};
-                }
-            } else if (*next_char == '\0') {
-                // end of string, no second zoom level given
-                expire_tiles_zoom = expire_tiles_zoom_min;
-            } else {
-                throw std::runtime_error{"Minimum and maximum zoom level for "
-                                         "tile expiry must be separated by "
-                                         "'-'."};
-            }
+        case 'e': // --expire-tiles
+            parse_expire_tiles_param(optarg, &expire_tiles_zoom_min,
+                                     &expire_tiles_zoom);
             break;
-        case 'o':
+        case 'o': // --expire-output
             expire_tiles_filename = optarg;
             break;
-        case 214:
+        case 214: // --expire-bbox-size
             expire_tiles_max_bbox = atof(optarg);
             break;
-        case 'O':
+        case 'O': // --output
             output_backend = optarg;
-            if (output_backend == "multi") {
-                log_warn("The 'multi' output is deprecated and will be removed."
-                         "Please switch to the 'flex' output.");
-            }
             break;
-        case 'x':
+        case 'x': // --extra-attributes
             extra_attributes = true;
             break;
-        case 'k':
+        case 'k': // --hstore
             if (hstore_mode != hstore_column::none) {
                 throw std::runtime_error{"You can not specify both --hstore "
                                          "(-k) and --hstore-all (-j)."};
             }
             hstore_mode = hstore_column::norm;
             break;
-        case 208:
+        case 208: // --hstore-match-only
             hstore_match_only = true;
             break;
-        case 'j':
+        case 'j': // --hstore-all
             if (hstore_mode != hstore_column::none) {
                 throw std::runtime_error{"You can not specify both --hstore "
                                          "(-k) and --hstore-all (-j)."};
             }
             hstore_mode = hstore_column::all;
             break;
-        case 'z':
+        case 'z': // --hstore-column
             hstore_columns.emplace_back(optarg);
             break;
-        case 'G':
+        case 'G': // --multi-geometry
             enable_multi = true;
             break;
-        case 'r':
+        case 'r': // --input-reader
             if (std::strcmp(optarg, "auto") != 0) {
                 input_format = optarg;
             }
             break;
-        case 'h':
+        case 'h': // --help
             m_print_help = true;
             break;
-        case 'I':
+        case 'I': // --disable-parallel-indexing
             parallel_indexing = false;
             break;
-        case 204:
-            if (std::strcmp(optarg, "dense") == 0) {
-                alloc_chunkwise = ALLOC_DENSE;
-            } else if (std::strcmp(optarg, "chunk") == 0) {
-                alloc_chunkwise = ALLOC_DENSE | ALLOC_DENSE_CHUNK;
-            } else if (std::strcmp(optarg, "sparse") == 0) {
-                alloc_chunkwise = ALLOC_SPARSE;
-            } else if (std::strcmp(optarg, "optimized") == 0) {
-                alloc_chunkwise = ALLOC_DENSE | ALLOC_SPARSE;
-            } else {
-                throw std::runtime_error{
-                    "Unrecognized cache strategy {}."_format(optarg)};
-            }
+        case 204: // -cache-strategy
+            log_warn("Deprecated option --cache-strategy ignored");
             break;
-        case 205:
-            num_procs = atoi(optarg);
-            if (num_procs < 1) {
-                log_warn("--number-processes must be at least 1. Using 1.");
-                num_procs = 1;
-            } else if (num_procs > 32) {
-                // The threads will open up database connections which will
-                // run out at some point. It depends on the number of tables
-                // how many connections there are. The number 32 is way beyond
-                // anything that will make sense here.
-                log_warn("--number-processes too large. Set to 32.");
-                num_procs = 32;
-            }
+        case 205: // --number-processes
+            num_procs = parse_number_processes_param(optarg);
             break;
-        case 206:
+        case 206: // --drop
             droptemp = true;
             break;
-        case 'F':
+        case 'F': // --flat-nodes
             flat_node_file = optarg;
             break;
-        case 211:
+        case 211: // --hstore-add-index
             enable_hstore_index = true;
             break;
-        case 212:
+        case 212: // --tag-transform-script
             tag_transform_script = optarg;
             break;
-        case 213:
+        case 213: // --reproject-area
             reproject_area = true;
             break;
-        case 'V':
-            fmt::print(stderr, "Compiled using the following library versions:\n");
-            fmt::print(stderr, "Libosmium {}\n", LIBOSMIUM_VERSION_STRING);
-            fmt::print(stderr, "Proj {}\n", get_proj_version());
-#ifdef HAVE_LUA
-#ifdef HAVE_LUAJIT
-            fmt::print(stderr, "{} ({})\n", LUA_RELEASE, LUAJIT_VERSION);
-#else
-            fmt::print(stderr, "{}\n", LUA_RELEASE);
-#endif
-#else
-            fmt::print(stderr, "Lua support not included\n");
-#endif
-            exit(EXIT_SUCCESS);
+        case 'V': // --version
+            print_version();
+            std::exit(EXIT_SUCCESS); // NOLINT(concurrency-mt-unsafe)
             break;
-        case 215:
+        case 215: // --middle-schema
             middle_dbschema = optarg;
+            check_identifier(middle_dbschema, "--middle-schema parameter");
             break;
-        case 216:
+        case 216: // --output-pgsql-schema
             output_dbschema = optarg;
+            check_identifier(output_dbschema, "--output-pgsql-schema parameter");
             break;
-        case 217:
-            if (std::strcmp(optarg, "false") == 0) {
-                with_forward_dependencies = false;
-            } else if (std::strcmp(optarg, "true") == 0) {
-                with_forward_dependencies = true;
-            } else {
-                throw std::runtime_error{
-                    "Unknown value for --with-forward-dependencies option: {}\n"_format(
-                        optarg)};
-            }
+        case 217: // --with-forward-dependencies=BOOL
+            with_forward_dependencies =
+                parse_with_forward_dependencies_param(optarg);
             break;
-        case 300:
+        case 300: // --middle-way-node-index-id-shift
             way_node_index_id_shift = atoi(optarg);
             break;
         case 400: // --log-level=LEVEL
-            if (std::strcmp(optarg, "debug") == 0) {
-                get_logger().set_level(log_level::debug);
-            } else if (std::strcmp(optarg, "info") == 0) {
-                get_logger().set_level(log_level::info);
-            } else if ((std::strcmp(optarg, "warn") == 0) ||
-                       (std::strcmp(optarg, "warning") == 0)) {
-                get_logger().set_level(log_level::warn);
-            } else if (std::strcmp(optarg, "error") == 0) {
-                get_logger().set_level(log_level::error);
-            } else {
-                throw std::runtime_error{
-                    "Unknown value for --log-level option: {}"_format(optarg)};
-            }
+            parse_log_level_param(optarg);
             break;
         case 401: // --log-progress=VALUE
-            if (std::strcmp(optarg, "true") == 0) {
-                get_logger().enable_progress();
-            } else if (std::strcmp(optarg, "false") == 0) {
-                get_logger().disable_progress();
-            } else if (std::strcmp(optarg, "auto") == 0) {
-                get_logger().auto_progress();
-            } else {
-                throw std::runtime_error{
-                    "Unknown value for --log-progress option: {}"_format(
-                        optarg)};
-            }
+            parse_log_progress_param(optarg);
             break;
         case 402: // --log-sql
             get_logger().enable_sql();
@@ -667,11 +682,10 @@ options_t::options_t(int argc, char *argv[]) : options_t()
     check_options();
 
     if (pass_prompt) {
-        char const *prompt = simple_prompt("Password:", 100, 0);
-        if (prompt != nullptr) {
-            database_options.password = prompt;
-        }
+        database_options.password = util::get_password();
     }
+
+    conninfo = build_conninfo(database_options);
 }
 
 void options_t::check_options()
@@ -711,7 +725,7 @@ void options_t::check_options()
     if (cache == 0) {
         if (!slim) {
             throw std::runtime_error{
-                "RAM node cache can only be disable in slim mode."};
+                "RAM node cache can only be disabled in slim mode."};
         }
         if (flat_node_file.empty()) {
             log_warn("RAM cache is disabled. This will likely slow down "
@@ -719,6 +733,10 @@ void options_t::check_options()
         }
     }
 
+    if (!slim && !flat_node_file.empty()) {
+        log_warn("Ignoring --flat-nodes/-F setting in non-slim mode");
+    }
+
     // zoom level 31 is the technical limit because we use 32-bit integers for the x and y index of a tile ID
     if (expire_tiles_zoom_min > 31) {
         expire_tiles_zoom_min = 31;
@@ -732,10 +750,21 @@ void options_t::check_options()
                  "large and has been set to 31.");
     }
 
+    if (expire_tiles_zoom != 0 && projection->target_srs() != 3857) {
+        log_warn("Expire has been enabled (with -e or --expire-tiles) but "
+                 "target SRS is not Mercator (EPSG:3857). Expire disabled!");
+        expire_tiles_zoom = 0;
+    }
+
     if (output_backend == "flex" || output_backend == "gazetteer") {
         if (style == DEFAULT_STYLE) {
             throw std::runtime_error{
                 "You have to set the config file with the -S|--style option."};
         }
     }
+
+    if (output_backend == "gazetteer") {
+        log_warn(
+            "The 'gazetteer' output is deprecated and will soon be removed.");
+    }
 }
