@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -30,9 +30,7 @@ enum class hstore_column : char
     all = 2
 };
 
-/**
- * Database options, not specific to a table
- */
+/// Database connection options.
 struct database_options_t
 {
     std::string db;
@@ -40,10 +38,10 @@ struct database_options_t
     std::string host;
     std::string password;
     std::string port;
-
-    std::string conninfo() const;
 };
 
+std::string build_conninfo(database_options_t const &opt);
+
 /**
  * Outputs can signal their requirements to the middle by setting these fields.
  */
@@ -83,6 +81,7 @@ public:
     /**
      * Constructor parsing the options from the command line.
      */
+    // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,hicpp-avoid-c-arrays,modernize-avoid-c-arrays)
     options_t(int argc, char *argv[]);
 
     /**
@@ -93,11 +92,9 @@ public:
         return m_print_help;
     }
 
+    std::string conninfo; ///< connection info for database
+
     std::string prefix{"planet_osm"};         ///< prefix for table names
-    std::shared_ptr<reprojection> projection; ///< SRS of projection
-    bool append = false;                      ///< Append to existing data
-    bool slim = false;                        ///< In slim mode
-    int cache = 800;                          ///< Memory usable for cache in MB
 
     /// Pg Tablespace to store indexes on main tables (no default TABLESPACE)
     std::string tblsmain_index{};
@@ -118,31 +115,54 @@ public:
     std::string output_dbschema{};
 
     std::string style{DEFAULT_STYLE}; ///< style file to use
-    uint32_t expire_tiles_zoom = 0;   ///< Zoom level for tile expiry list
 
-    /// Minimum zoom level for tile expiry list
-    uint32_t expire_tiles_zoom_min = 0;
+    /// Name of the flat node file used. Empty if flat node file is not enabled.
+    std::string flat_node_file{};
 
-    /// Max bbox size in either dimension to expire full bbox for a polygon
-    double expire_tiles_max_bbox = 20000.0;
+    std::string tag_transform_script;
 
     /// File name to output expired tiles list to
     std::string expire_tiles_filename{"dirty_tiles"};
 
-    /// add an additional hstore column with objects key/value pairs, and what type of hstore column
-    hstore_column hstore_mode = hstore_column::none;
+    std::string output_backend{"pgsql"};
+    std::string input_format; ///< input file format (default: autodetect)
 
-    bool enable_hstore_index = false; ///< add an index on the hstore column
-
-    /// Output multi-geometries intead of several simple geometries
-    bool enable_multi = false;
+    osmium::Box bbox;
 
     /// list of columns that should be written into their own hstore column
     std::vector<std::string> hstore_columns;
 
-    bool keep_coastlines = false;
-    bool parallel_indexing = true;
+    std::vector<std::string> input_files;
+
+    std::shared_ptr<reprojection> projection; ///< SRS of projection
+
+    /// Max bbox size in either dimension to expire full bbox for a polygon
+    double expire_tiles_max_bbox = 20000.0;
+
+    uint32_t expire_tiles_zoom = 0; ///< Zoom level for tile expiry list
+
+    /// Minimum zoom level for tile expiry list
+    uint32_t expire_tiles_zoom_min = 0;
+
+    int cache = 800; ///< Memory usable for cache in MB
+
     unsigned int num_procs;
+
+    /**
+     * How many bits should the node id be shifted for the way node index?
+     * The result is a lossy index which is significantly smaller.
+     * See https://osm2pgsql.org/doc/manual.html#bucket-index-for-slim-mode
+     * Use 0 to use a classic loss-less GIN index.
+     */
+    uint8_t way_node_index_id_shift = 5;
+
+    /// add an additional hstore column with objects key/value pairs, and what type of hstore column
+    hstore_column hstore_mode = hstore_column::none;
+
+    bool append = false;                      ///< Append to existing data
+    bool slim = false;                        ///< In slim mode
+    bool extra_attributes = false;
+    bool keep_coastlines = false;
     bool droptemp = false; ///< drop slim mode temp tables after act
 
     /**
@@ -154,32 +174,17 @@ public:
     /// only copy rows that match an explicitly listed key
     bool hstore_match_only = false;
 
+    bool enable_hstore_index = false; ///< add an index on the hstore column
+
+    /// Output multi-geometries intead of several simple geometries
+    bool enable_multi = false;
+
     bool reproject_area = false;
 
-    /// Name of the flat node file used. Empty if flat node file is not enabled.
-    std::string flat_node_file{};
-
-    std::string tag_transform_script;
-
+    bool parallel_indexing = true;
     bool create = false;
     bool pass_prompt = false;
 
-    database_options_t database_options;
-    std::string output_backend{"pgsql"};
-    std::string input_format; ///< input file format (default: autodetect)
-    osmium::Box bbox;
-    bool extra_attributes = false;
-
-    std::vector<std::string> input_files;
-
-    /**
-     * How many bits should the node id be shifted for the way node index?
-     * Use 0 to disable for backwards compatibility.
-     * Currently the default is 0, making osm2pgsql backwards compatible to
-     * earlier versions.
-     */
-    uint8_t way_node_index_id_shift = 0;
-
 private:
 
     bool m_print_help = false;
