@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -16,9 +16,9 @@
 
 #ifdef HAVE_LUA
 # include "output-flex.hpp"
-# define flex_backend "flex, "
+static constexpr char const *const flex_backend = "flex, ";
 #else
-# define flex_backend ""
+static constexpr char const *const flex_backend = "";
 #endif
 
 #include <stdexcept>
@@ -30,24 +30,21 @@ output_t::create_output(std::shared_ptr<middle_query_t> const &mid,
                         std::shared_ptr<thread_pool_t> thread_pool,
                         options_t const &options)
 {
-    auto copy_thread =
-        std::make_shared<db_copy_thread_t>(options.database_options.conninfo());
-
     if (options.output_backend == "pgsql") {
         return std::make_shared<output_pgsql_t>(mid, std::move(thread_pool),
-                                                options, copy_thread);
+                                                options);
     }
 
 #ifdef HAVE_LUA
     if (options.output_backend == "flex") {
         return std::make_shared<output_flex_t>(mid, std::move(thread_pool),
-                                               options, copy_thread);
+                                               options);
     }
 #endif
 
     if (options.output_backend == "gazetteer") {
         return std::make_shared<output_gazetteer_t>(mid, std::move(thread_pool),
-                                                    options, copy_thread);
+                                                    options);
     }
 
     if (options.output_backend == "null") {
@@ -55,10 +52,9 @@ output_t::create_output(std::shared_ptr<middle_query_t> const &mid,
                                                options);
     }
 
-    throw std::runtime_error{
-        "Output backend '{}' not recognised. Should be one "
-        "of [pgsql, " flex_backend
-        "gazetteer, null]."_format(options.output_backend)};
+    throw fmt_error("Output backend '{}' not recognised. Should be one of"
+                    " [pgsql, {}gazetteer, null].",
+                    options.output_backend, flex_backend);
 }
 
 output_t::output_t(std::shared_ptr<middle_query_t> mid,
@@ -66,14 +62,14 @@ output_t::output_t(std::shared_ptr<middle_query_t> mid,
                    options_t const &options)
 : m_mid(std::move(mid)), m_options(&options),
   m_thread_pool(std::move(thread_pool))
+{}
 
+output_t::output_t(output_t const *other, std::shared_ptr<middle_query_t> mid)
+: m_mid(std::move(mid)), m_options(other->m_options),
+  m_thread_pool(other->m_thread_pool),
+  m_output_requirements(other->m_output_requirements)
 {}
 
 output_t::~output_t() = default;
 
-void output_t::free_middle_references()
-{
-    m_mid.reset();
-}
-
-void output_t::merge_expire_trees(output_t *) {}
+void output_t::free_middle_references() { m_mid.reset(); }
