@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -33,7 +33,7 @@ idlist_t get_ids_from_db(pg_conn_t const *db_connection, char const *stmt,
 void create_geom_check_trigger(pg_conn_t *db_connection,
                                std::string const &schema,
                                std::string const &table,
-                               std::string const &geom_column);
+                               std::string const &condition);
 
 void drop_geom_check_trigger(pg_conn_t *db_connection,
                              std::string const &schema,
@@ -42,4 +42,11 @@ void drop_geom_check_trigger(pg_conn_t *db_connection,
 void analyze_table(pg_conn_t const &db_connection, std::string const &schema,
                    std::string const &name);
 
+/**
+ * Check whether the table with the specified name exists in the specified
+ * schema in the database. Leave schema empty to check in the 'public' schema.
+ */
+bool has_table(pg_conn_t const &db_connection, std::string const &schema,
+               std::string const &table);
+
 #endif // OSM2PGSQL_PGSQL_HELPER_HPP
