@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -30,13 +30,13 @@ public:
     virtual std::unique_ptr<tagtransform_t> clone() const = 0;
 
     virtual bool filter_tags(osmium::OSMObject const &o, bool *polygon,
-                             bool *roads, taglist_t &out_tags) = 0;
+                             bool *roads, taglist_t *out_tags) = 0;
 
     virtual bool filter_rel_member_tags(taglist_t const &rel_tags,
                                         osmium::memory::Buffer const &members,
                                         rolelist_t const &member_roles,
                                         bool *make_boundary, bool *make_polygon,
-                                        bool *roads, taglist_t &out_tags) = 0;
+                                        bool *roads, taglist_t *out_tags) = 0;
 };
 
 #endif // OSM2PGSQL_TAGTRANSFORM_HPP
