@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -17,6 +17,7 @@
 #include <cstring>
 #include <stdexcept>
 #include <string>
+#include <string_view>
 #include <utility>
 
 namespace ewkb {
@@ -298,25 +299,23 @@ private:
  * Empty Multi* geometries and Geometry Collections will return a NULL
  * geometry object.
  *
- * Call next() to get a pointer to the next character that is not part of the
- * geometry any more. If this is not the same as the end pointer given to
- * the constructor, this means that there is extra data available in the
- * input data.
+ * Call is_done() to check whether the parser read all available data.
  *
  * Can only parse (E)WKB in native byte order.
  */
 class ewkb_parser_t
 {
 public:
-    ewkb_parser_t(char const *it, char const *end)
-    : m_it(it), m_end(end), m_max_length((end - it) / (sizeof(double) * 2))
+    ewkb_parser_t(std::string_view input)
+    : m_data(input),
+      m_max_length(static_cast<uint32_t>(input.size()) / (sizeof(double) * 2))
     {}
 
     geom::geometry_t operator()()
     {
         geom::geometry_t geom;
 
-        if (m_it == m_end) {
+        if (m_data.empty()) {
             return geom;
         }
 
@@ -345,19 +344,19 @@ public:
             parse_collection(&geom);
             break;
         default:
-            throw std::runtime_error{
-                "Invalid WKB geometry: Unknown geometry type {}"_format(type)};
+            throw fmt_error("Invalid WKB geometry: Unknown geometry type {}",
+                            type);
         }
 
         return geom;
     }
 
-    char const *next() const noexcept { return m_it; }
+    bool is_done() const noexcept { return m_data.empty(); }
 
 private:
     void check_bytes(uint32_t bytes) const
     {
-        if (static_cast<std::size_t>(m_end - m_it) < bytes) {
+        if (m_data.size() < bytes) {
             throw std::runtime_error{"Invalid WKB geometry: Incomplete"};
         }
     }
@@ -367,8 +366,8 @@ private:
         check_bytes(sizeof(uint32_t));
 
         uint32_t data = 0;
-        std::memcpy(&data, m_it, sizeof(uint32_t));
-        m_it += sizeof(uint32_t);
+        std::memcpy(&data, m_data.data(), sizeof(uint32_t));
+        m_data.remove_prefix(sizeof(uint32_t));
 
         return data;
     }
@@ -392,7 +391,7 @@ private:
 
     uint32_t parse_header(geom::geometry_t *geom = nullptr)
     {
-        if (static_cast<uint8_t>(*m_it++) !=
+        if (static_cast<uint8_t>(m_data.front()) !=
             ewkb::wkb_byte_order_type_t::Endian) {
             throw std::runtime_error
             {
@@ -406,6 +405,7 @@ private:
                 "osm2pgsql can only process geometries in native byte order."
             };
         }
+        m_data.remove_prefix(1);
 
         auto type = parse_uint32();
         if (type & ewkb::geometry_type::wkb_srid) {
@@ -428,8 +428,8 @@ private:
         check_bytes(sizeof(double) * 2);
 
         std::array<double, 2> data{};
-        std::memcpy(&data[0], m_it, sizeof(double) * 2);
-        m_it += sizeof(double) * 2;
+        std::memcpy(data.data(), m_data.data(), sizeof(double) * 2);
+        m_data.remove_prefix(sizeof(double) * 2);
 
         point->set_x(data[0]);
         point->set_y(data[1]);
@@ -439,10 +439,9 @@ private:
     {
         auto const num_points = parse_length();
         if (num_points < min_points) {
-            throw std::runtime_error{
-                "Invalid WKB geometry: {} are not"
-                " enough points (must be at least {})"_format(num_points,
-                                                              min_points)};
+            throw fmt_error("Invalid WKB geometry: {} are not"
+                            " enough points (must be at least {})",
+                            num_points, min_points);
         }
         points->reserve(num_points);
         for (uint32_t i = 0; i < num_points; ++i) {
@@ -479,9 +478,9 @@ private:
             auto &point = multipoint.add_geometry();
             uint32_t const type = parse_header();
             if (type != geometry_type::wkb_point) {
-                throw std::runtime_error{
-                    "Invalid WKB geometry: Multipoint containing"
-                    " something other than point: {}"_format(type)};
+                throw fmt_error("Invalid WKB geometry: Multipoint containing"
+                                " something other than point: {}",
+                                type);
             }
             parse_point(&point);
         }
@@ -501,9 +500,10 @@ private:
             auto &linestring = multilinestring.add_geometry();
             uint32_t const type = parse_header();
             if (type != geometry_type::wkb_line) {
-                throw std::runtime_error{
+                throw fmt_error(
                     "Invalid WKB geometry: Multilinestring containing"
-                    " something other than linestring: {}"_format(type)};
+                    " something other than linestring: {}",
+                    type);
             }
             parse_point_list(&linestring, 2);
         }
@@ -523,9 +523,9 @@ private:
             auto &polygon = multipolygon.add_geometry();
             uint32_t const type = parse_header();
             if (type != geometry_type::wkb_polygon) {
-                throw std::runtime_error{
-                    "Invalid WKB geometry: Multipolygon containing"
-                    " something other than polygon: {}"_format(type)};
+                throw fmt_error("Invalid WKB geometry: Multipolygon containing"
+                                " something other than polygon: {}",
+                                type);
             }
             parse_polygon(&polygon);
         }
@@ -542,14 +542,13 @@ private:
 
         collection.reserve(num_geoms);
         for (uint32_t i = 0; i < num_geoms; ++i) {
-            ewkb_parser_t parser{m_it, m_end};
+            ewkb_parser_t parser{m_data};
             collection.add_geometry(parser());
-            m_it = parser.next();
+            m_data = parser.m_data;
         }
     }
 
-    char const *m_it;
-    char const *m_end;
+    std::string_view m_data;
     uint32_t m_max_length;
 
 }; // class ewkb_parser_t
@@ -564,13 +563,12 @@ std::string geom_to_ewkb(geom::geometry_t const &geom, bool ensure_multi)
         static_cast<uint32_t>(geom.srid()), ensure_multi});
 }
 
-geom::geometry_t ewkb_to_geom(std::string const &wkb)
+geom::geometry_t ewkb_to_geom(std::string_view wkb)
 {
-    const char *const end = wkb.data() + wkb.size();
-    ewkb::ewkb_parser_t parser{wkb.data(), end};
+    ewkb::ewkb_parser_t parser{wkb};
     auto geom = parser();
 
-    if (parser.next() != end) {
+    if (!parser.is_done()) {
         throw std::runtime_error{"Invalid WKB geometry: Extra data at end"};
     }
 
