@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -22,10 +22,10 @@ using copy_mgr_t = db_copy_mgr_t<db_deleter_by_id_t>;
 
 static std::shared_ptr<db_target_descr_t> setup_table(std::string const &cols)
 {
-    auto conn = db.connect();
+    auto const conn = db.connect();
     conn.exec("DROP TABLE IF EXISTS test_copy_mgr");
-    conn.exec("CREATE TABLE test_copy_mgr (id int8{}{})"_format(
-        cols.empty() ? "" : ",", cols));
+    conn.exec("CREATE TABLE test_copy_mgr (id int8{}{})",
+              cols.empty() ? "" : ",", cols);
 
     auto table = std::make_shared<db_target_descr_t>();
     table->name = "test_copy_mgr";
@@ -35,53 +35,53 @@ static std::shared_ptr<db_target_descr_t> setup_table(std::string const &cols)
 }
 
 template <typename... ARGS>
-void add_row(copy_mgr_t &mgr, std::shared_ptr<db_target_descr_t> const &t,
-             ARGS &&... args)
+void add_row(copy_mgr_t *mgr, std::shared_ptr<db_target_descr_t> const &t,
+             ARGS &&...args)
 {
-    mgr.new_line(t);
-    mgr.add_columns(std::forward<ARGS>(args)...);
-    mgr.finish_line();
+    mgr->new_line(t);
+    mgr->add_columns(std::forward<ARGS>(args)...);
+    mgr->finish_line();
 
-    mgr.sync();
+    mgr->sync();
 }
 
 template <typename T>
-void add_array(copy_mgr_t &mgr, std::shared_ptr<db_target_descr_t> const &t,
+void add_array(copy_mgr_t *mgr, std::shared_ptr<db_target_descr_t> const &t,
                int id, std::vector<T> const &values)
 {
-    mgr.new_line(t);
-    mgr.add_column(id);
-    mgr.new_array();
+    mgr->new_line(t);
+    mgr->add_column(id);
+    mgr->new_array();
     for (auto const &v : values) {
-        mgr.add_array_elem(v);
+        mgr->add_array_elem(v);
     }
-    mgr.finish_array();
-    mgr.finish_line();
+    mgr->finish_array();
+    mgr->finish_line();
 
-    mgr.sync();
+    mgr->sync();
 }
 
 static void
-add_hash(copy_mgr_t &mgr, std::shared_ptr<db_target_descr_t> const &t, int id,
+add_hash(copy_mgr_t *mgr, std::shared_ptr<db_target_descr_t> const &t, int id,
          std::vector<std::pair<std::string, std::string>> const &values)
 {
-    mgr.new_line(t);
+    mgr->new_line(t);
 
-    mgr.add_column(id);
-    mgr.new_hash();
-    for (auto const &v : values) {
-        mgr.add_hash_elem(v.first, v.second);
+    mgr->add_column(id);
+    mgr->new_hash();
+    for (auto const &[k, v] : values) {
+        mgr->add_hash_elem(k, v);
     }
-    mgr.finish_hash();
-    mgr.finish_line();
+    mgr->finish_hash();
+    mgr->finish_line();
 
-    mgr.sync();
+    mgr->sync();
 }
 
 static void check_row(std::vector<std::string> const &row)
 {
-    auto conn = db.connect();
-    auto res = conn.require_row("SELECT * FROM test_copy_mgr");
+    auto const conn = db.connect();
+    auto const res = conn.require_row("SELECT * FROM test_copy_mgr");
 
     for (std::size_t i = 0; i < row.size(); ++i) {
         CHECK(res.get_value(0, (int)i) == row[i]);
@@ -90,11 +90,11 @@ static void check_row(std::vector<std::string> const &row)
 
 TEST_CASE("copy_mgr_t")
 {
-    copy_mgr_t mgr(std::make_shared<db_copy_thread_t>(db.conninfo()));
+    copy_mgr_t mgr{std::make_shared<db_copy_thread_t>(db.conninfo())};
 
     SECTION("Insert null")
     {
-        auto t = setup_table("big int8, t text");
+        auto const t = setup_table("big int8, t text");
 
         mgr.new_line(t);
         mgr.add_column(0);
@@ -103,8 +103,8 @@ TEST_CASE("copy_mgr_t")
         mgr.finish_line();
         mgr.sync();
 
-        auto conn = db.connect();
-        auto res = conn.require_row("SELECT * FROM test_copy_mgr");
+        auto const conn = db.connect();
+        auto const res = conn.require_row("SELECT * FROM test_copy_mgr");
 
         CHECK(res.is_null(0, 1));
         CHECK(res.is_null(0, 2));
@@ -112,61 +112,61 @@ TEST_CASE("copy_mgr_t")
 
     SECTION("Insert numbers")
     {
-        auto t = setup_table("big int8, small smallint");
+        auto const t = setup_table("big int8, small smallint");
 
-        add_row(mgr, t, 34, 0xfff12345678ULL, -4457);
+        add_row(&mgr, t, 34, 0xfff12345678ULL, -4457);
         check_row({"34", "17588196497016", "-4457"});
     }
 
     SECTION("Insert strings")
     {
-        auto t = setup_table("s0 text, s1 varchar");
+        auto const t = setup_table("s0 text, s1 varchar");
 
         SECTION("Simple strings")
         {
-            add_row(mgr, t, -2, "foo", "l");
+            add_row(&mgr, t, -2, "foo", "l");
             check_row({"-2", "foo", "l"});
         }
 
         SECTION("Strings with special characters")
         {
-            add_row(mgr, t, -2, "va\tr", "meme\n");
+            add_row(&mgr, t, -2, "va\tr", "meme\n");
             check_row({"-2", "va\tr", "meme\n"});
         }
 
         SECTION("Strings with more special characters")
         {
-            add_row(mgr, t, -2, "\rrun", "K\\P");
+            add_row(&mgr, t, -2, "\rrun", "K\\P");
             check_row({"-2", "\rrun", "K\\P"});
         }
 
         SECTION("Strings with space and quote")
         {
-            add_row(mgr, t, 1, "with space", "name \"quoted\"");
+            add_row(&mgr, t, 1, "with space", "name \"quoted\"");
             check_row({"1", "with space", "name \"quoted\""});
         }
     }
 
     SECTION("Insert int arrays")
     {
-        auto t = setup_table("a int[]");
+        auto const t = setup_table("a int[]");
 
-        add_array<int>(mgr, t, -9000, {45, -2, 0, 56});
+        add_array<int>(&mgr, t, -9000, {45, -2, 0, 56});
         check_row({"-9000", "{45,-2,0,56}"});
     }
 
     SECTION("Insert string arrays")
     {
-        auto t = setup_table("a text[]");
+        auto const t = setup_table("a text[]");
 
-        add_array<std::string>(mgr, t, 3,
+        add_array<std::string>(&mgr, t, 3,
                                {"foo", "", "with space", "with \"quote\"",
                                 "the\t", "line\nbreak", "rr\rrr", "s\\l"});
         check_row({"3", "{foo,\"\",\"with space\",\"with "
                         "\\\"quote\\\"\",\"the\t\",\"line\nbreak\","
                         "\"rr\rrr\",\"s\\\\l\"}"});
 
-        auto c = db.connect();
+        auto const c = db.connect();
         CHECK(c.result_as_string("SELECT a[4] FROM test_copy_mgr") ==
               "with \"quote\"");
         CHECK(c.result_as_string("SELECT a[5] FROM test_copy_mgr") == "the\t");
@@ -178,7 +178,7 @@ TEST_CASE("copy_mgr_t")
 
     SECTION("Insert hashes")
     {
-        auto t = setup_table("h hstore");
+        auto const t = setup_table("h hstore");
 
         std::vector<std::pair<std::string, std::string>> const values = {
             {"one", "two"},           {"key 1", "value 1"},
@@ -186,14 +186,55 @@ TEST_CASE("copy_mgr_t")
             {"key\n3", "value\n3"},   {"key\r4", "value\r4"},
             {"key\\5", "value\\5"}};
 
-        add_hash(mgr, t, 42, values);
+        add_hash(&mgr, t, 42, values);
 
-        auto c = db.connect();
+        auto const c = db.connect();
 
-        for (auto const &v : values) {
+        for (auto const &[k, v] : values) {
             auto const res = c.result_as_string(
-                "SELECT h->'{}' FROM test_copy_mgr"_format(v.first));
-            CHECK(res == v.second);
+                fmt::format("SELECT h->'{}' FROM test_copy_mgr", k));
+            CHECK(res == v);
         }
     }
+
+    SECTION("Insert something and roll back")
+    {
+        auto const t = setup_table("t text");
+
+        mgr.new_line(t);
+        mgr.add_column(0);
+        mgr.add_column("foo");
+        mgr.rollback_line();
+        mgr.sync();
+
+        auto const conn = db.connect();
+        CHECK(conn.get_count("test_copy_mgr") == 0);
+    }
+
+    SECTION("Insert something, insert more, roll back, insert something else")
+    {
+        auto const t = setup_table("t text");
+
+        mgr.new_line(t);
+        mgr.add_column(0);
+        mgr.add_column("good");
+        mgr.finish_line();
+
+        mgr.new_line(t);
+        mgr.add_column(1);
+        mgr.add_column("bad");
+        mgr.rollback_line();
+
+        mgr.new_line(t);
+        mgr.add_column(2);
+        mgr.add_column("better");
+        mgr.finish_line();
+        mgr.sync();
+
+        auto const conn = db.connect();
+        auto const res = conn.exec("SELECT t FROM test_copy_mgr ORDER BY id");
+        CHECK(res.num_tuples() == 2);
+        CHECK(res.get(0, 0) == "good");
+        CHECK(res.get(1, 0) == "better");
+    }
 }
