@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -95,7 +95,7 @@ TEMPLATE_TEST_CASE("middle import", "", options_slim_default,
                    options_slim_with_schema, options_ram_optimized)
 {
     options_t const options = TestType::options(db);
-    testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
+    testing::cleanup::file_t const flatnode_cleaner{options.flat_node_file};
 
     auto conn = db.connect();
     auto const num_tables =
@@ -160,8 +160,8 @@ TEMPLATE_TEST_CASE("middle import", "", options_slim_default,
         // set nodes
         for (osmid_t i = 1; i <= 10; ++i) {
             nds.push_back(i);
-            auto const &node = buffer.add_node("n{} x{:.7f} y{:.7f}"_format(
-                i, lon - i * 0.003, lat + i * 0.001));
+            auto const &node = buffer.add_node(fmt::format(
+                "n{} x{:.7f} y{:.7f}", i, lon - i * 0.003, lat + i * 0.001));
             mid->node(node);
         }
         mid->after_nodes();
@@ -213,6 +213,14 @@ TEMPLATE_TEST_CASE("middle import", "", options_slim_default,
         // set the relation
         auto const &relation =
             buffer.add_relation("r123 Mw11@,w10@outer,n1@,w12@inner");
+
+        std::vector<std::pair<osmium::item_type, osmid_t>> const expected = {
+            {osmium::item_type::way, 11},
+            {osmium::item_type::way, 10},
+            {osmium::item_type::node, 1},
+            {osmium::item_type::way, 12},
+        };
+
         osmium::CRC<osmium::CRC_zlib> orig_crc;
         orig_crc.update(relation);
 
@@ -232,17 +240,62 @@ TEMPLATE_TEST_CASE("middle import", "", options_slim_default,
         crc.update(rel);
         CHECK(orig_crc().checksum() == crc().checksum());
 
-        // retrieve the supporting ways
-        REQUIRE(mid_q->rel_members_get(rel, &outbuf,
+        // retrieve node members only
+        osmium::memory::Buffer memberbuf{
+            4096, osmium::memory::Buffer::auto_grow::yes};
+        REQUIRE(mid_q->rel_members_get(rel, &memberbuf,
+                                       osmium::osm_entity_bits::node) == 1);
+
+        {
+            auto const objects = memberbuf.select<osmium::OSMObject>();
+            auto it = objects.cbegin();
+            auto const end = objects.cend();
+            for (auto const &p : expected) {
+                if (p.first == osmium::item_type::node) {
+                    REQUIRE(it != end);
+                    REQUIRE(it->type() == p.first);
+                    REQUIRE(it->id() == p.second);
+                    ++it;
+                }
+            }
+        }
+
+        memberbuf.clear();
+
+        // retrieve way members only
+        REQUIRE(mid_q->rel_members_get(rel, &memberbuf,
                                        osmium::osm_entity_bits::way) == 3);
 
-        for (auto &w : outbuf.select<osmium::Way>()) {
-            REQUIRE(w.id() >= 10);
-            REQUIRE(w.id() <= 12);
-            auto const &expected = nds[w.id() - 10];
-            REQUIRE(w.nodes().size() == expected.size());
-            for (size_t i = 0; i < expected.size(); ++i) {
-                REQUIRE(w.nodes()[i].ref() == expected[i]);
+        {
+            auto const objects = memberbuf.select<osmium::OSMObject>();
+            auto it = objects.cbegin();
+            auto const end = objects.cend();
+            for (auto const &p : expected) {
+                if (p.first == osmium::item_type::way) {
+                    REQUIRE(it != end);
+                    REQUIRE(it->type() == p.first);
+                    REQUIRE(it->id() == p.second);
+                    ++it;
+                }
+            }
+        }
+
+        memberbuf.clear();
+
+        // retrieve all members
+        REQUIRE(mid_q->rel_members_get(rel, &memberbuf,
+                                       osmium::osm_entity_bits::node |
+                                           osmium::osm_entity_bits::way) == 4);
+
+        {
+            auto const objects = memberbuf.select<osmium::OSMObject>();
+            auto it = objects.cbegin();
+            auto const end = objects.cend();
+            for (auto const &p : expected) {
+                REQUIRE(it != end);
+                REQUIRE(it->type() == p.first);
+                REQUIRE(it->id() == p.second);
+                ++it;
             }
         }
 
@@ -292,7 +345,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update node", "",
 
     options_t options = TestType::options(db);
 
-    testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
+    testing::cleanup::file_t const flatnode_cleaner{options.flat_node_file};
 
     // Prepare a buffer with some nodes which we will add and change.
     test_buffer_t buffer;
@@ -486,7 +539,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update way", "",
 
     options_t options = TestType::options(db);
 
-    testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
+    testing::cleanup::file_t const flatnode_cleaner{options.flat_node_file};
 
     // Create some ways we'll use for the tests.
     test_buffer_t buffer;
@@ -636,7 +689,7 @@ TEMPLATE_TEST_CASE("middle: add way with attributes", "", options_slim_default,
     SECTION("With attributes") { options.extra_attributes = true; }
     SECTION("No attributes") { options.extra_attributes = false; }
 
-    testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
+    testing::cleanup::file_t const flatnode_cleaner{options.flat_node_file};
 
     // Create some ways we'll use for the tests.
     test_buffer_t buffer;
@@ -719,7 +772,7 @@ TEMPLATE_TEST_CASE("middle: add, delete and update relation", "",
 
     options_t options = TestType::options(db);
 
-    testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
+    testing::cleanup::file_t const flatnode_cleaner{options.flat_node_file};
 
     // Create some relations we'll use for the tests.
     test_buffer_t buffer;
@@ -865,7 +918,7 @@ TEMPLATE_TEST_CASE("middle: add relation with attributes", "",
     SECTION("With attributes") { options.extra_attributes = true; }
     SECTION("No attributes") { options.extra_attributes = false; }
 
-    testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
+    testing::cleanup::file_t const flatnode_cleaner{options.flat_node_file};
 
     // Create some relations we'll use for the tests.
     test_buffer_t buffer;
@@ -914,7 +967,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
 
     options_t options = TestType::options(db);
 
-    testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
+    testing::cleanup::file_t const flatnode_cleaner{options.flat_node_file};
 
     // create some nodes and ways we'll use for the tests
     test_buffer_t buffer;
@@ -937,7 +990,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in way", "", options_slim_default,
     // closed properly.
     {
         auto mid = std::make_shared<middle_pgsql_t>(thread_pool, &options);
-        full_dependency_manager_t dependency_manager{mid};
+        full_dependency_manager_t const dependency_manager{mid};
         mid->start();
 
         mid->node(node10);
@@ -1051,7 +1104,7 @@ TEMPLATE_TEST_CASE("middle: change nodes in relation", "", options_slim_default,
 
     options_t options = TestType::options(db);
 
-    testing::cleanup::file_t flatnode_cleaner{options.flat_node_file};
+    testing::cleanup::file_t const flatnode_cleaner{options.flat_node_file};
 
     // create some nodes, ways, and relations we'll use for the tests
     test_buffer_t buffer;
