@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -12,7 +12,6 @@
 #include <vector>
 
 #include "common-import.hpp"
-#include "config.h"
 #include "reprojection.hpp"
 
 static testing::db::import_t db;
@@ -74,7 +73,8 @@ TEST_CASE("Projection setup")
 
     option_params.push_back("foo");
 
-    options_t options{(int)option_params.size(), (char **)option_params.data()};
+    options_t const options{(int)option_params.size(),
+                            (char **)option_params.data()};
 
     if (!proj_name.empty()) {
         CHECK(options.projection->target_desc() == proj_name);
