@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -24,7 +24,9 @@ static testing::db::import_t db;
 static char const *const data_file = "liechtenstein-2013-08-03.osm.pbf";
 
 std::vector<std::string> get_files() {
+    // NOLINTNEXTLINE(concurrency-mt-unsafe)
     char const *env = std::getenv("EXAMPLE_FILES");
+    REQUIRE(env);
     return osmium::split_string(env, ',', true);
 }
 
