@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -12,6 +12,8 @@
 #include "common-import.hpp"
 #include "common-options.hpp"
 
+#include <array>
+
 static testing::db::import_t db;
 
 TEST_CASE("compute Z order")
@@ -21,13 +23,16 @@ TEST_CASE("compute Z order")
 
     auto conn = db.db().connect();
 
-    char const *expected[] = {"motorway", "trunk", "primary", "secondary",
-                              "tertiary"};
+    std::array<char const *, 5> const expected = {
+        "motorway", "trunk", "primary", "secondary", "tertiary"};
 
     for (unsigned i = 0; i < 5; ++i) {
-        auto const sql = "SELECT highway FROM osm2pgsql_test_line"
-                         " WHERE layer IS NULL ORDER BY z_order DESC"
-                         " LIMIT 1 OFFSET {}"_format(i);
+        auto const sql = fmt::format("SELECT highway"
+                                     " FROM osm2pgsql_test_line"
+                                     " WHERE layer IS NULL"
+                                     " ORDER BY z_order DESC"
+                                     " LIMIT 1 OFFSET {}",
+                                     i);
         REQUIRE(expected[i] == conn.result_as_string(sql));
     }
 
