@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -34,27 +34,20 @@ TEST_CASE("Table name with schema")
     REQUIRE(qualified_name("osm", "foo") == R"("osm"."foo")");
 }
 
-TEST_CASE("PostGIS version")
-{
-    auto conn = db.db().connect();
-    auto const postgis_version = get_postgis_version(conn);
-    REQUIRE(postgis_version.major >= 2);
-}
-
 TEST_CASE("query with SELECT should work")
 {
     auto conn = db.db().connect();
-    auto const result = conn.query(PGRES_TUPLES_OK, "SELECT 42");
+    auto const result = conn.exec("SELECT 42");
     REQUIRE(result.status() == PGRES_TUPLES_OK);
     REQUIRE(result.num_fields() == 1);
     REQUIRE(result.num_tuples() == 1);
-    REQUIRE(result.get_value_as_string(0, 0) == "42");
+    REQUIRE(result.get(0, 0) == "42");
 }
 
 TEST_CASE("query with invalid SQL should fail")
 {
     auto conn = db.db().connect();
-    REQUIRE_THROWS(conn.query(PGRES_TUPLES_OK, "NOT-VALID-SQL"));
+    REQUIRE_THROWS(conn.exec("NOT-VALID-SQL"));
 }
 
 TEST_CASE("exec with invalid SQL should fail")
@@ -72,7 +65,7 @@ TEST_CASE("exec_prepared without parameters should work")
     REQUIRE(result.status() == PGRES_TUPLES_OK);
     REQUIRE(result.num_fields() == 1);
     REQUIRE(result.num_tuples() == 1);
-    REQUIRE(result.get_value_as_string(0, 0) == "42");
+    REQUIRE(result.get(0, 0) == "42");
 }
 
 TEST_CASE("exec_prepared with single string parameters should work")
@@ -84,19 +77,23 @@ TEST_CASE("exec_prepared with single string parameters should work")
     REQUIRE(result.status() == PGRES_TUPLES_OK);
     REQUIRE(result.num_fields() == 1);
     REQUIRE(result.num_tuples() == 1);
-    REQUIRE(result.get_value_as_string(0, 0) == "17");
+    REQUIRE(result.get(0, 0) == "17");
 }
 
 TEST_CASE("exec_prepared with string parameters should work")
 {
     auto conn = db.db().connect();
-    conn.exec("PREPARE test(int, int, int) AS SELECT $1 + $2 + $3");
+    conn.exec("PREPARE test(int, int, int, int, int)"
+              " AS SELECT $1 + $2 + $3 + $4 + $5");
 
-    auto const result = conn.exec_prepared("test", "1", "2", std::string{"3"});
+    std::string a{"4"}; // NOLINT(misc-const-correctness)
+    std::string const b{"5"};
+    auto const result =
+        conn.exec_prepared("test", "1", "2", std::string{"3"}, a, b);
     REQUIRE(result.status() == PGRES_TUPLES_OK);
     REQUIRE(result.num_fields() == 1);
     REQUIRE(result.num_tuples() == 1);
-    REQUIRE(result.get_value_as_string(0, 0) == "6");
+    REQUIRE(result.get(0, 0) == "15");
 }
 
 TEST_CASE("exec_prepared with non-string parameters should work")
@@ -108,5 +105,5 @@ TEST_CASE("exec_prepared with non-string parameters should work")
     REQUIRE(result.status() == PGRES_TUPLES_OK);
     REQUIRE(result.num_fields() == 1);
     REQUIRE(result.num_tuples() == 1);
-    REQUIRE(result.get_value_as_string(0, 0) == "6");
+    REQUIRE(result.get(0, 0) == "6");
 }
